local LSP = LibStub:GetLibrary("LibSimcraftParser")
if not LSP then return end

local SpellInfo = {}

LibSimcraftParserTooltip:HookScript("OnTooltipSetSpell", function(self)
	local id = select(3,self:GetSpell())
	if not id then
		LibSimcraftParserTooltip.spellid = nil
		return
	else
		LibSimcraftParserTooltip.spellid = select(3,self:GetSpell()) or nil
	end
end)

LSP.GetSpellDB = function(useSupplementalSpellStore, spellBlacklistTable)  -- Loading the supplemental DB incurs a performance cost. This flag will be temporarily enabled during simcraft import.
	local lReturnDB = {}
	local function Initialize(spells)
		if getmetatable(spells) then return end     -- Already initialized
		local function addUnlessExists(table, key, value)
			if key and not rawget(table, key) then
				rawset(table, key, value)
			end
		end
		local function getAliases(spellID, spellName)
			local name = spellName or GetSpellInfo(spellID)
			local link = GetSpellLink(spellID)
			local linkName, simcraftName
			if name then
				linkName = '[' .. name .. ']'
				simcraftName = strlower(string.gsub(name:gsub('%s','_'):gsub('-','_'), '[^%w_]', ''))    -- Simcraft spell names ignore non-alphanumeric characters except spaces and dashes, which become underscores
			end
			return name, link, linkName, simcraftName
		end
		local function addAliases(spellID, spellInfo)
			if tonumber(spellID) then
				local name, link, linkName, simcraftName = getAliases(spellID, spellInfo.spellname)
				addUnlessExists(spells, name, spellInfo)
				addUnlessExists(spells, linkName, spellInfo)    -- Create wowace deserialized link pointer
				addUnlessExists(spells, simcraftName, spellInfo)    -- Create simcraft spell pointer
				addUnlessExists(spells, link, spellInfo)     -- Create link pointer
			end
		end
		local unknownKeyHandler = {
			-- Fires on assignment table[key] = value when table[key] is nil
			__newindex = function(table, key, value)
				rawset(table, key, value)
				addAliases(key, value)
			end,
			-- Fires on access table[key] when table[key] is nil
			__index = function(table, key)
				if not key then return end
				rawset(table, key, false)
				if GetSpellInfo(key) then
					LSP.SetSpellInfo(lReturnDB, spellBlacklistTable, key)
				elseif useSupplementalSpellStore and not tonumber(key) then       -- Could be attempt to get non-spellbook spell by name
					local spellIDs = LSP.GetSupplementalSpellInfo(key)
					if spellIDs then
						LSP.SetSpellInfo(lReturnDB, spellBlacklistTable, spellIDs[#spellIDs])     -- TODO: Investigate handling multiple candidate IDs
					end
				end
				return rawget(table, key)
			end
		}
		return setmetatable(spells, unknownKeyHandler)
	end
	local function InitializeSpellsFromSource(spells, getName, setTooltip, sourceBook)
		local spellCount
		if type(spells) == 'table' then
			for id, record in pairs(spells) do
				LSP.SetSpellInfo(lReturnDB, spellBlacklistTable, id, nil, nil, nil, nil, record)
			end
			return
		else
			spellCount = tonumber(spells)
		end
		if not spellCount or spellCount < 1 then return end
		for i = 1, spellCount do
			local lSpell = getName(i, sourceBook)
			if ( not lSpell ) then break end
			if setTooltip then
				LibSimcraftParserTooltip:SetOwner(UIParent, "ANCHOR_NONE")
				setTooltip(LibSimcraftParserTooltip, i, sourceBook)
				lSpell = LibSimcraftParserTooltip.spellid or lSpell
				LibSimcraftParserTooltip:Hide()
			end
			LSP.SetSpellInfo(lReturnDB, spellBlacklistTable, lSpell, nil, nil, i, sourceBook)
		end
	end

	Initialize(lReturnDB)
	local _, _, offset, spellCount = GetSpellTabInfo(GetNumSpellTabs())
	InitializeSpellsFromSource(offset + spellCount, GetSpellBookItemName, LibSimcraftParserTooltip.SetSpellBookItem, BOOKTYPE_SPELL)
	-- HasPetSpells() returns pet spell count if player has a pet; else nil
	InitializeSpellsFromSource(HasPetSpells(), GetSpellBookItemName, LibSimcraftParserTooltip.SetSpellBookItem, BOOKTYPE_PET)
	InitializeSpellsFromSource(GetNumTalents(), GetTalentInfo, LibSimcraftParserTooltip.SetTalent)
	InitializeSpellsFromSource(LSP.GetSpellInfoForClass( select(2, UnitClass("player") ) ) )
	return lReturnDB
end

function LSP.SetSpellInfo(spelldb, spellBlacklistTable, spell, spellnametext, simcraftname, spellbookindex, spellbooktype, infoRecord)
	local stored = rawget(spelldb, spell) -- rawget forces an exact match (no possible aliases are searched)
	if stored and stored.spellid and not infoRecord then 
		return stored.spellid 
	end
	local info = {}
	if spellBlacklistTable and spellBlacklistTable[tostring(spell)] then return nil end
	if tonumber(spell) and infoRecord then -- Initialize from SimulationCraft DB
		local record = LSP.Decompress(infoRecord)
		info.spellid = spell
		info.spellname = GetSpellInfo(spell)
		info.baseduration = record.duration
		info.replacementid = record.replacesSpell
		info.triggeredbyid = record.triggeredBy
		info.triggersid = record.triggersSpell
		info.baseticktime = record.tickTime
		info.isaura = record.isAura
		info.iseffect = record.isEffect
		info.schools = record.spellSchools
		info.texture = GetSpellTexture(info.spellname)

		if spellBlacklistTable and spellBlacklistTable[tostring(info.spellid)] then return nil end
		if spellBlacklistTable and spellBlacklistTable[tostring(info.spellname)] then return nil end

		if not stored then
			local related = rawget(spelldb, info.spellname) -- exact match on spellname pointer
			if related then
				related.related = related.related or {}
				tinsert(related.related, spell)
				info.related = { related.spellid }
				-- rawset won't add aliased pointers, since presumably 'related' already has
				rawset(spelldb, spell, info)
			else
				spelldb[spell] = info
			end
		else
			for k, v in pairs(info) do
				stored[k] = v
			end
		end
	else
		local lSpellID = tonumber(spell) or strmatch(GetSpellLink(spell) or '', "spell:(%d+)")
		if ( LSP.IsBlank( lSpellID ) ) then
			_,lSpellID = GetSpellBookItemInfo(spell or '')
		end
		if ( LSP.IsBlank( lSpellID ) ) then
			-- print("  trying spellbook using index " )
			_,lSpellID = GetSpellBookItemInfo(spellbookindex or 0, spellbooktype or BOOKTYPE_SPELL)
		end
		-- print("  after trying spellbook lSpellID="..tostring(lSpellID) )
		-- print("  lSpellID3="..tostring(lSpellID) )
		local lSpellIDKey = tostring(lSpellID or '')
		if ( not lSpellID ) then return nil end -- Exit if we could not find the spellID
		if spellBlacklistTable and spellBlacklistTable[tostring(lSpellID)] then return nil end
		-- See if this ID is already in spelldb
		local extant = rawget(spelldb, lSpellIDKey)
		if extant then
			rawset(spelldb, spell, extant)
			return lSpellIDKey
		end

		info.spellid = lSpellIDKey
		spellnametext = spellnametext or GetSpellInfo( lSpellID )-- Try to get the spellnametext
		-- JC 9/28: Check if old behavior relied upon (adding a "blank" spell not recognized by GetSpellInfo to spelldb)
		if ( not spellnametext ) then return nil end -- Exit if we could not get the text of the spell
		if spellBlacklistTable and spellBlacklistTable[tostring(spellnametext)] then return nil end
		-- print("  SetSpellInfo spell="..tostring(spell) )
		-- print("  spellnametext="..tostring(spellnametext) )
		lTexture = GetSpellTexture(spellnametext)
		-- print("  lTexture="..tostring(lTexture) )
		if ( not lTexture and spellbookindex and  spellbooktype) then
			lTexture = GetSpellTexture(spellbookindex, spellbooktype)
			-- print("  lTexturetry2="..tostring(lTexture) )
		end
		info.texture = lTexture
		-- print("SetSpellInfo spell="..tostring(spell) )
		-- print("spellnametext="..tostring(spellnametext) )
		info.spellname = spellnametext
		spelldb[lSpellIDKey] = info
	end
	stored = spelldb[spell]
	if stored and stored.spellid then 
		if not stored.texture then
			stored.texture = GetSpellTexture(tonumber(stored.spellid))
		end
		return stored.spellid 
	end
end

-- Generated on 2013-11-02

LSP.GetSpellInfoForClass = function(class)
  local choice = string.lower(class:gsub('%A', ''))
  index = SpellInfo['GetClassSpells_' .. choice]
  return index and index()
end
LSP.GetSpellInfoForRace = function(race)
  local choice = string.lower(race:gsub('%A', ''))
  index = SpellInfo['GetRacialSpells_' .. choice]
  return index and index()
end
LSP.GetSupplementalSpellInfo = function(name)
  if not name or name == '' then return end
  local alias = name:gsub('%s','_'):gsub('-','_'):gsub('[^%w_]',''):lower()
  local get = SpellInfo['GetUnknown_' .. alias:sub(1, 1)]
  local spells = get and get()
  if not spells or not spells[alias] then return end
  local result = {}
  for i, id in pairs(spells[alias]) do
    result[i] = id
  end
  return result
end

function SpellInfo.GetRacialSpells_draenei()
 return {
["121093"]={},
["132295"]={},
["28878"]={},
["28880"]={},
["59221"]={},
["59535"]={},
["59536"]={},
["59538"]={},
["59539"]={},
["59540"]={},
["59541"]={},
["59542"]={},
["59543"]={},
["59544"]={},
["59545"]={},
["59547"]={},
["59548"]={},
["6562"]={},
}
end

function SpellInfo.GetRacialSpells_bloodelf()
 return {
["129597"]={},
["25046"]={},
["28730"]={},
["50613"]={},
["69179"]={},
["80483"]={},
["822"]={},
}
end

function SpellInfo.GetRacialSpells_worgen()
 return {
["68975"]={},
["68976"]={},
["68992"]={},
["94293"]={},
}
end

function SpellInfo.GetRacialSpells_orc()
 return {
["20572"]={},
["20573"]={},
["20574"]={},
["20575"]={},
["20576"]={},
["21563"]={},
["33697"]={},
["33702"]={},
["54562"]={},
["65222"]={},
}
end

function SpellInfo.GetRacialSpells_gnome()
 return {
["20589"]={},
["20591"]={},
["20592"]={},
["92680"]={},
}
end

function SpellInfo.GetRacialSpells_troll()
 return {
["20555"]={},
["20557"]={},
["26290"]={},
["26297"]={},
["58943"]={},
}
end

function SpellInfo.GetRacialSpells_tauren()
 return {
["20549"]={},
["20550"]={},
["20551"]={},
}
end

function SpellInfo.GetRacialSpells_dwarf()
 return {
["20594"]={},
["20595"]={},
["20596"]={},
["59224"]={},
["92682"]={},
}
end

function SpellInfo.GetRacialSpells_undead()
 return {
["20577"]={},
["20579"]={},
["5227"]={},
["7744"]={},
}
end

function SpellInfo.GetRacialSpells_nightelf()
 return {
["20582"]={},
["20583"]={},
["21009"]={},
["58984"]={},
}
end

function SpellInfo.GetRacialSpells_human()
 return {
["20597"]={},
["20598"]={},
["20864"]={},
["59752"]={},
}
end

function SpellInfo.GetRacialSpells_pandaren()
 return { }
end

function SpellInfo.GetRacialSpells_goblin()
 return {
["69041"]={},
["69042"]={},
["69070"]={},
}
end

function SpellInfo.GetClassSpells_warrior()
 return {
["100"]={b="126664",s=1,},
["100130"]={b="115804",s=1,},
["102060"]={d=4000,s=1,},
["103840"]={["?"]=1,s=1,},
["105771"]={["?"]=1,d=4000,s=1,},
["107566"]={d=5000,s=1,},
["107570"]={["?"]=1,b="145585",s=1,},
["107574"]={d=24000,s=1,},
["108126"]={["?"]=1,d=6000,s=1,},
["10840"]={s=1,},
["10841"]={s=1,},
["112048"]={d=6000,s=1,},
["113344"]={["?"]=1,d=6000,s=1,t=1000,},
["113746"]={d=30000,s=1,},
["114028"]={d=5000,s=1,},
["114029"]={["?"]=1,b="46947",d=10000,s=1,},
["114030"]={d=12000,s=1,},
["114192"]={["?"]=1,d=30000,s=1,t=1000,},
["114198"]={["?"]=1,d=6000,s=1,},
["114203"]={d=15000,s=1,},
["114207"]={["?"]=1,d=10000,s=1,},
["115317"]={d=6000,s=1,},
["115767"]={d=15000,s=1,t=3000,},
["115798"]={d=30000,s=1,},
["115804"]={d=10000,s=1,},
["115945"]={d=10000,s=2,},
["115993"]={d=5000,s=1,},
["1160"]={["?"]=1,b="125565",d=10000,s=1,},
["118000"]={["?"]=1,s=1,},
["118038"]={d=8000,s=1,},
["118340"]={["?"]=1,d=6000,s=1,},
["118779"]={["?"]=1,d=6000,s=1,},
["118895"]={["?"]=1,d=500,s=1,},
["121186"]={d=1000,s=1,},
["122016"]={d=10000,s=1,},
["122475"]={s=1,},
["122510"]={d=10000,s=1,},
["12292"]={["?"]=1,d=12000,s=1,},
["12294"]={b="115804",d=10000,s=1,},
["12323"]={d=15000,s=1,},
["12328"]={d=10000,s=1,},
["124184"]={d=11000,s=1,},
["125565"]={["?"]=1,a=1,d=11000,s=1,u="1160",},
["126661"]={d=3000,s=1,t=100,},
["127123"]={s=1,},
["12880"]={d=6000,s=1,},
["12968"]={d=15000,s=1,},
["12975"]={d=20000,s=1,},
["129923"]={d=8000,s=1,},
["131116"]={d=12000,s=1,},
["132168"]={["?"]=1,d=4000,s=1,},
["132169"]={["?"]=1,d=4000,s=1,},
["132365"]={d=20000,s=1,},
["132404"]={["?"]=1,d=6000,s=1,},
["137637"]={["?"]=1,d=15000,s=1,},
["138127"]={["?"]=1,d=10000,s=1,},
["138279"]={["?"]=1,d=15000,s=1,},
["139958"]={d=10000,s=1,},
["144500"]={d=10000,s=1,},
["145585"]={s=1,},
["145674"]={d=20000,s=1,},
["1464"]={s=1,},
["147297"]={d=6000,s=1,t=1000,},
["147367"]={d=600000,s=1,},
["147531"]={["?"]=1,d=6000,s=1,},
["147833"]={["?"]=1,d=10000,s=1,},
["1680"]={b="44949",s=1,},
["1715"]={d=15000,s=1,},
["1719"]={d=12000,s=1,},
["18498"]={d=3000,s=1,},
["18499"]={d=6000,s=1,},
["18629"]={s=1,},
["18630"]={s=1,},
["20243"]={b="113746",r="7386",s=1,},
["20572"]={d=15000,s=1,},
["23881"]={s=1,},
["23920"]={d=5000,s=1,},
["23922"]={s=1,},
["2457"]={d=-1,s=1,},
["2458"]={d=-1,s=1,},
["2565"]={["?"]=1,s=1,},
["28880"]={["?"]=1,d=15000,s=2,t=3000,},
["29842"]={d=10000,s=1,t=2000,},
["32216"]={["?"]=1,d=20000,s=1,},
["32747"]={d=3000,s=1,},
["3276"]={s=1,},
["3277"]={s=1,},
["3278"]={s=1,},
["3411"]={["?"]=1,s=1,},
["34428"]={["?"]=1,s=1,},
["355"]={d=3000,s=1,},
["40274"]={s=1,},
["41312"]={s=1,},
["46114"]={s=1,},
["46115"]={s=1,},
["469"]={b="92049",d=300000,s=1,},
["46916"]={b="100130",d=15000,s=1,},
["46924"]={b="50622",d=6000,s=1,t=1000,},
["46947"]={["?"]=1,a=1,d=6000,s=1,u="114029",},
["46968"]={["?"]=1,s=1,},
["50227"]={d=5000,s=1,},
["5246"]={d=8000,s=1,},
["5302"]={d=6000,s=1,},
["5308"]={s=1,},
["55694"]={d=5000,s=1,t=1000,},
["56480"]={s=1,},
["56483"]={s=1,},
["57755"]={s=1,},
["60503"]={b="7384",d=12000,s=1,},
["61483"]={s=1,},
["6343"]={s=1,},
["64382"]={d=10000,s=1,},
["6544"]={b="52174",d=1000,s=1,},
["6552"]={d=4000,s=1,},
["6572"]={s=1,},
["6673"]={b="92049",d=300000,s=1,},
["676"]={d=8000,s=1,},
["68992"]={d=10000,s=1,},
["69179"]={d=2000,s=64,},
["71"]={d=-1,s=1,},
["7384"]={s=1,},
["7386"]={b="113746",s=1,},
["78"]={s=1,},
["7922"]={d=1500,s=1,},
["7928"]={s=1,},
["7929"]={s=1,},
["7934"]={s=1,},
["81326"]={d=30000,s=1,},
["81714"]={s=1,},
["845"]={s=1,},
["84619"]={d=5000,s=1,},
["85288"]={b="96103",s=1,},
["85739"]={d=10000,s=1,},
["86346"]={["?"]=1,d=6500,s=1,},
["86663"]={d=20000,s=1,},
["871"]={d=12000,s=1,},
["92049"]={d=6000,s=1,},
["97462"]={["?"]=1,s=1,},
["97463"]={["?"]=1,d=10000,s=1,},
}
end

function SpellInfo.GetClassSpells_paladin()
 return {
["1022"]={d=10000,s=2,},
["10326"]={d=40000,s=2,},
["1038"]={d=10000,s=2,},
["1044"]={d=6000,s=2,},
["105361"]={d=-1,s=2,},
["105593"]={d=6000,s=2,},
["105742"]={d=15000,s=2,},
["105809"]={d=18000,s=1,},
["10840"]={s=1,},
["10841"]={s=1,},
["110300"]={d=2000,s=1,},
["114039"]={d=6000,s=2,},
["114157"]={["?"]=1,s=2,},
["114158"]={["?"]=1,b="122773",s=1,},
["114163"]={d=30000,s=2,t=3000,},
["114165"]={["?"]=1,s=2,},
["114232"]={d=-1,s=2,},
["114250"]={d=15000,s=2,},
["114637"]={b="85673",d=20000,s=1,},
["114852"]={["?"]=1,s=2,},
["114871"]={["?"]=1,s=2,},
["114916"]={["?"]=1,d=10000,s=2,t=1000,},
["114917"]={d=10000,s=2,t=1000,},
["114919"]={d=2000,s=2,},
["115522"]={d=6000,s=2,},
["115547"]={d=20000,s=2,t=2000,},
["115654"]={d=15000,s=2,},
["115668"]={d=6000,s=2,},
["115750"]={["?"]=1,d=6000,s=2,},
["115752"]={["?"]=1,d=3000,s=2,},
["115798"]={d=30000,s=1,},
["115995"]={d=4000,s=1,},
["119072"]={d=3000,s=2,},
["121027"]={d=10000,s=1,},
["121467"]={d=12000,s=1,},
["121783"]={s=2,},
["122032"]={["?"]=1,s=2,},
["122287"]={s=8,},
["122773"]={["?"]=1,a=1,d=15500,s=1,u="114158",},
["127122"]={s=1,},
["127123"]={s=1,},
["132365"]={d=20000,s=1,},
["132403"]={["?"]=1,d=3000,s=2,},
["136494"]={["?"]=1,s=2,},
["138162"]={["?"]=1,d=6000,s=2,},
["138242"]={d=5000,s=1,},
["138250"]={["?"]=1,d=10000,s=32,},
["144581"]={d=10000,s=2,t=1000,},
["144587"]={d=6000,s=2,},
["144595"]={d=12000,s=2,},
["144622"]={d=20000,s=1,},
["144624"]={d=15000,s=2,},
["18629"]={s=1,},
["18630"]={s=1,},
["19740"]={d=3600000,s=2,},
["19750"]={s=2,},
["20066"]={d=60000,s=2,},
["20154"]={d=-1,s=2,},
["20164"]={["?"]=1,b="20170",d=-1,s=2,},
["20165"]={b="20167",d=-1,s=2,},
["20170"]={["?"]=1,a=1,d=8000,s=2,u="20164",},
["20217"]={d=3600000,s=2,},
["20271"]={s=2,},
["20473"]={s=2,},
["20925"]={["?"]=1,d=30000,s=2,t=6000,},
["24275"]={s=2,},
["25771"]={d=60000,s=2,},
["25780"]={d=-1,s=0,},
["26573"]={["?"]=1,b="82366",d=9000,s=2,t=1000,},
["2812"]={d=4000,s=2,},
["28730"]={d=2000,s=64,},
["28880"]={["?"]=1,d=15000,s=2,t=3000,},
["31801"]={d=-1,s=2,},
["31803"]={d=15000,s=2,t=3000,},
["31821"]={["?"]=1,b="64364",d=6000,s=1,},
["31842"]={d=20000,s=2,},
["31850"]={d=10000,s=1,},
["31884"]={d=20000,s=2,},
["31935"]={d=3000,s=2,},
["32747"]={d=3000,s=1,},
["3276"]={s=1,},
["3277"]={s=1,},
["3278"]={s=1,},
["35395"]={s=1,},
["40274"]={s=1,},
["41311"]={s=1,},
["41312"]={s=1,},
["46107"]={s=1,},
["46114"]={s=1,},
["46115"]={s=1,},
["498"]={["?"]=1,d=10000,s=2,},
["4987"]={s=2,},
["53385"]={s=2,},
["53563"]={b="53651",d=-1,s=2,},
["53595"]={b="88263",s=1,},
["53600"]={["?"]=1,s=2,},
["53651"]={d=-1,s=2,},
["54149"]={d=15000,s=2,},
["54428"]={d=9000,s=2,t=3000,},
["54957"]={d=7000,s=2,},
["54986"]={s=2,},
["55694"]={d=5000,s=1,t=1000,},
["56480"]={s=1,},
["56483"]={s=1,},
["59542"]={["?"]=1,d=15000,s=2,t=3000,},
["59578"]={d=6000,s=1,},
["61483"]={s=1,},
["62124"]={d=3000,s=2,},
["62271"]={s=1,},
["633"]={s=2,},
["635"]={s=2,},
["63529"]={d=10000,s=2,},
["642"]={d=8000,s=2,},
["64364"]={["?"]=1,a=1,d=6000,s=2,u="31821",},
["65148"]={["?"]=1,d=6000,s=2,},
["68992"]={d=10000,s=1,},
["6940"]={d=12000,s=2,},
["7928"]={s=1,},
["7929"]={s=1,},
["7934"]={s=1,},
["81326"]={d=30000,s=1,},
["81714"]={s=1,},
["81715"]={s=1,},
["82326"]={s=2,},
["82327"]={s=2,},
["82366"]={["?"]=1,a=1,d=10000,s=2,u="26573",},
["84963"]={d=20000,s=2,t=2000,},
["85222"]={s=2,},
["85256"]={s=1,},
["853"]={d=6000,s=2,},
["85416"]={b="98057",d=6000,s=1,},
["85673"]={["?"]=1,s=2,},
["86273"]={d=15000,s=2,},
["86657"]={d=40000,s=2,},
["86659"]={["?"]=1,d=12000,s=2,},
["86669"]={["?"]=1,d=15000,s=2,},
["86678"]={s=2,},
["86698"]={["?"]=1,b="86760",d=30000,s=2,},
["86700"]={d=30000,s=2,},
["879"]={["?"]=1,s=2,},
["88819"]={d=10000,s=1,},
["90174"]={d=8000,s=2,},
["94686"]={d=8000,s=1,},
["96231"]={d=4000,s=1,},
}
end

function SpellInfo.GetClassSpells_shaman()
 return {
["105763"]={d=15000,s=8,},
["105779"]={d=20000,s=8,},
["105821"]={d=4000,s=8,},
["105869"]={d=-1,s=8,},
["105873"]={d=-1,s=1,},
["105877"]={d=15000,s=8,},
["1064"]={s=8,},
["108269"]={d=6000,s=8,},
["108270"]={d=30000,s=8,},
["108271"]={d=6000,s=1,},
["108273"]={d=6000,s=8,},
["108280"]={d=11000,s=8,},
["108281"]={d=10000,s=1,},
["108285"]={s=1,},
["108287"]={d=1000,s=8,},
["10840"]={s=1,},
["10841"]={s=1,},
["114049"]={["?"]=1,s=1,},
["114051"]={["?"]=1,b="114089",d=15000,s=1,},
["114074"]={["?"]=1,s=4,},
["114738"]={["?"]=1,s=4,},
["114893"]={d=10000,s=8,},
["115356"]={["?"]=1,b="115357",d=15000,s=8,},
["115357"]={["?"]=1,e=1,s=8,u="115356",},
["115798"]={d=30000,s=1,},
["117014"]={["?"]=1,s=28,},
["117588"]={["?"]=1,s=4,},
["118297"]={d=21000,s=4,t=3000,},
["118337"]={d=10000,s=8,},
["118345"]={d=4000,s=1,},
["118347"]={d=-1,s=8,},
["118350"]={d=-1,s=4,},
["118470"]={["?"]=1,d=10000,s=4,},
["118472"]={["?"]=1,d=8000,s=8,},
["118473"]={["?"]=1,d=10000,s=8,},
["118475"]={["?"]=1,d=5000,s=8,},
["118522"]={["?"]=1,d=8000,s=28,},
["118905"]={d=5000,s=8,},
["119523"]={["?"]=1,d=6000,s=8,},
["120588"]={["?"]=1,s=28,},
["120668"]={["?"]=1,d=10000,s=8,},
["120676"]={["?"]=1,d=-1,s=8,},
["127120"]={s=1,},
["127121"]={s=1,},
["138146"]={s=8,},
["142912"]={d=6000,s=1,},
["144964"]={d=10000,s=1,},
["144967"]={d=10000,s=1,},
["144999"]={d=2000,s=1,},
["145000"]={d=10000,s=1,},
["145002"]={s=8,},
["145385"]={d=15000,s=1,},
["145394"]={b="145385",d=15000,s=1,},
["1535"]={["?"]=1,s=4,},
["16166"]={d=20000,s=8,},
["16188"]={d=-1,s=1,},
["16190"]={d=16000,s=8,},
["16191"]={b="39610",d=-1,s=8,t=2000,},
["16246"]={d=15000,s=8,},
["16278"]={d=15000,s=1,},
["17364"]={["?"]=1,b="32175",d=15000,r="73899",s=1,},
["18629"]={s=1,},
["18630"]={s=1,},
["2008"]={b="148037",s=8,},
["2062"]={d=60000,s=8,},
["2484"]={d=20000,s=8,},
["26364"]={["?"]=1,e=1,s=8,u="324",},
["2645"]={d=-1,s=8,},
["2825"]={d=40000,s=8,},
["28880"]={["?"]=1,d=15000,s=2,t=3000,},
["2894"]={d=60000,s=4,},
["30823"]={d=15000,s=1,},
["31616"]={d=10000,s=8,},
["32175"]={["?"]=1,e=1,s=1,u="17364",},
["32182"]={d=40000,s=8,},
["324"]={["?"]=1,b="26364",d=3600000,s=8,},
["3276"]={s=1,},
["3277"]={s=1,},
["3278"]={s=1,},
["331"]={s=8,},
["33697"]={d=15000,s=1,},
["3599"]={d=60000,s=4,},
["3600"]={d=5000,s=8,},
["3606"]={s=4,},
["36213"]={s=8,},
["370"]={s=8,},
["379"]={["?"]=1,s=8,},
["403"]={["?"]=1,s=8,},
["41314"]={s=1,},
["41315"]={s=1,},
["41316"]={s=1,},
["421"]={s=8,},
["45284"]={["?"]=1,s=8,},
["46110"]={s=1,},
["46112"]={s=1,},
["46113"]={s=1,},
["51485"]={d=20000,s=8,},
["51490"]={d=5000,s=8,},
["51505"]={["?"]=1,s=4,},
["51514"]={d=60000,s=8,},
["51533"]={d=30000,s=8,},
["51730"]={s=8,},
["51886"]={s=8,},
["51945"]={d=12000,s=8,t=3000,},
["52127"]={b="52128",d=3600000,s=8,},
["53390"]={d=15000,s=1,},
["53817"]={d=30000,s=1,},
["5394"]={["?"]=1,d=15000,s=8,},
["56487"]={s=1,},
["56574"]={s=1,},
["57723"]={d=600000,s=1,},
["57724"]={d=600000,s=1,},
["57984"]={s=4,},
["57994"]={d=3000,s=8,},
["58859"]={s=8,},
["58875"]={["?"]=1,b="58876",d=15000,s=1,},
["58876"]={["?"]=1,a=1,d=100,s=1,u="58875",},
["59547"]={["?"]=1,d=15000,s=2,t=3000,},
["60103"]={s=4,},
["61295"]={d=18000,s=8,t=3000,},
["61482"]={s=1,},
["61882"]={["?"]=1,d=10000,s=8,t=1000,},
["6196"]={d=60000,s=8,},
["63685"]={d=5000,s=16,},
["64695"]={d=5000,s=8,},
["68992"]={d=10000,s=1,},
["73680"]={s=8,},
["73681"]={d=12000,s=1,},
["73682"]={d=5000,s=16,},
["73683"]={d=8000,s=4,},
["73684"]={d=5000,s=8,},
["73685"]={d=10000,s=8,},
["73899"]={s=1,},
["73920"]={d=10000,s=8,t=2000,},
["76780"]={d=50000,s=8,},
["77130"]={r="51886",s=8,},
["77451"]={["?"]=1,s=4,},
["77472"]={s=8,},
["77478"]={["?"]=1,d=3000,s=1,},
["77661"]={d=15000,s=4,},
["79206"]={d=15000,s=8,},
["7928"]={s=1,},
["7929"]={s=1,},
["7934"]={s=1,},
["8004"]={s=8,},
["8017"]={s=8,},
["8024"]={s=4,},
["8033"]={s=16,},
["8034"]={d=8000,s=16,},
["8042"]={b="115798",s=8,},
["8050"]={d=30000,s=4,t=3000,},
["8056"]={d=8000,s=16,},
["8143"]={d=6000,s=8,},
["81716"]={s=1,},
["81720"]={s=1,},
["8177"]={d=15000,s=8,},
["8190"]={d=60000,s=4,},
["8232"]={s=8,},
["88765"]={s=1,},
["974"]={["?"]=1,d=600000,s=8,},
["98008"]={d=6000,s=8,},
["98017"]={b="98020",d=10000,s=8,t=1000,},
}
end

function SpellInfo.GetClassSpells_monk()
 return {
["100780"]={s=1,},
["100784"]={["?"]=1,s=1,},
["100787"]={b="125359",s=1,},
["101545"]={["?"]=1,d=2000,s=1,},
["101546"]={["?"]=1,b="107270",d=2250,s=1,t=750,},
["101643"]={["?"]=1,b="102141",s=1,},
["102141"]={["?"]=1,a=1,d=900000,s=1,u="101643",},
["103985"]={d=-1,s=1,},
["107428"]={["?"]=1,b="115804",s=1,},
["10840"]={s=1,},
["10841"]={s=1,},
["108561"]={d=-1,s=1,},
["109132"]={s=1,},
["113656"]={["?"]=1,b="120086",d=4000,s=1,t=1000,},
["115008"]={d=1100,s=1,},
["115069"]={d=-1,s=1,},
["115070"]={d=-1,s=1,},
["115072"]={["?"]=1,s=8,},
["115073"]={["?"]=1,s=4,},
["115078"]={d=40000,s=1,},
["115080"]={s=1,},
["115098"]={["?"]=1,s=8,},
["115129"]={["?"]=1,s=8,},
["115151"]={["?"]=1,s=8,},
["115175"]={["?"]=1,d=8000,s=8,t=1000,},
["115176"]={["?"]=1,b="131523",d=8000,s=8,},
["115180"]={["?"]=1,b="116330",s=1,},
["115181"]={["?"]=1,s=4,},
["115203"]={["?"]=1,s=1,},
["115213"]={d=6000,s=1,t=1000,},
["115288"]={d=6000,s=1,t=1000,},
["115294"]={["?"]=1,a=1,d=3000,s=1,t=500,u="115867",},
["115295"]={["?"]=1,d=30000,s=1,},
["115307"]={d=6000,s=1,},
["115308"]={["?"]=1,a=1,d=1000,s=1,u="128939",},
["115310"]={s=8,},
["115313"]={d=900000,s=1,},
["115315"]={d=900000,s=1,},
["115450"]={["?"]=1,s=1,},
["115460"]={d=60000,s=8,},
["115543"]={s=1,},
["115546"]={s=1,},
["115697"]={d=-1,s=1,},
["115798"]={d=30000,s=1,},
["115804"]={d=10000,s=1,},
["115867"]={["?"]=1,b="115294",d=120000,s=1,t=1000,},
["115921"]={["?"]=1,s=1,},
["116033"]={d=10000,s=1,},
["116087"]={d=30000,s=1,},
["116095"]={["?"]=1,d=15000,s=1,t=1000,},
["116330"]={["?"]=1,a=1,b="118022",d=15000,s=1,u="115180",},
["116670"]={s=8,},
["116680"]={d=30000,s=8,},
["116694"]={["?"]=1,s=8,},
["116705"]={["?"]=1,d=4000,s=1,},
["116706"]={["?"]=1,d=8000,s=1,},
["116709"]={["?"]=1,d=2000,s=1,},
["116740"]={["?"]=1,a=1,d=15000,s=1,u="125195",},
["116768"]={d=15000,s=1,},
["116781"]={d=3600000,s=1,},
["116844"]={b="137461",d=8000,s=8,},
["116847"]={b="148187",d=6000,s=8,t=750,},
["116849"]={d=12000,s=8,},
["117032"]={d=30000,s=1,},
["117368"]={d=8000,s=1,},
["117640"]={["?"]=1,s=8,},
["117666"]={["?"]=1,d=3600000,s=1,},
["117952"]={["?"]=1,d=6000,s=8,t=1000,},
["117953"]={["?"]=1,d=8000,s=2,},
["118022"]={e=1,s=1,u="116330",},
["118585"]={d=8000,s=1,},
["118604"]={["?"]=1,d=30000,s=1,},
["118636"]={d=30000,s=1,},
["118674"]={d=30000,s=1,},
["118864"]={d=15000,s=1,},
["119031"]={d=30000,s=1,},
["119381"]={d=5000,s=1,},
["119392"]={["?"]=1,b="125084",d=3000,s=1,},
["119582"]={s=1,},
["119611"]={["?"]=1,d=18000,s=8,t=2000,},
["119650"]={d=-1,s=1,},
["119996"]={d=2000,s=1,},
["120086"]={["?"]=1,a=1,d=4000,s=1,u="113656",},
["120273"]={d=15000,s=1,},
["120954"]={["?"]=1,d=20000,s=1,},
["121093"]={["?"]=1,d=15000,s=2,t=3000,},
["121253"]={s=1,},
["121286"]={d=120000,s=1,},
["122057"]={["?"]=1,s=1,},
["122242"]={["?"]=1,d=4000,s=1,},
["122278"]={d=45000,s=1,},
["122465"]={d=2500,s=1,},
["122470"]={["?"]=1,d=10000,s=1,},
["122783"]={d=6000,s=8,},
["123154"]={d=-1,s=1,},
["123231"]={d=15000,s=1,},
["123232"]={d=15000,s=1,},
["123273"]={["?"]=1,s=8,},
["123407"]={["?"]=1,d=2000,s=4,},
["123586"]={["?"]=1,d=4000,s=1,},
["123725"]={["?"]=1,d=8000,s=4,t=2000,},
["123727"]={["?"]=1,b="118022",d=15000,s=1,},
["123761"]={["?"]=1,s=1,},
["123904"]={d=45000,s=8,},
["123986"]={d=1000,s=8,},
["123999"]={b="123996",d=-1,s=8,t=1000,},
["124081"]={d=16000,s=8,t=2000,},
["124255"]={d=-1,s=1,t=1000,},
["124273"]={d=10000,s=1,},
["124280"]={["?"]=1,d=6000,s=8,t=1000,},
["124682"]={["?"]=1,s=8,},
["125084"]={["?"]=1,e=1,s=1,u="119392",},
["125170"]={["?"]=1,s=1,},
["125195"]={["?"]=1,b="116740",d=120000,s=1,t=1000,},
["125359"]={d=20000,s=1,},
["125883"]={d=-1,s=8,},
["125950"]={["?"]=1,d=8000,s=8,t=1000,},
["125953"]={["?"]=1,s=8,},
["126050"]={d=5000,s=1,},
["126119"]={d=-1,s=8,},
["126892"]={b="126896",s=8,},
["126893"]={d=-1,s=1,},
["126895"]={["?"]=1,s=8,},
["126896"]={["?"]=1,d=-1,s=1,},
["127118"]={s=1,},
["127119"]={s=1,},
["127722"]={d=30000,s=1,},
["128531"]={["?"]=1,d=4000,s=1,t=1000,},
["128939"]={["?"]=1,b="115308",d=30000,s=1,},
["129597"]={d=2000,s=64,},
["129914"]={d=-1,s=1,},
["130320"]={["?"]=1,d=15000,s=1,},
["131523"]={["?"]=1,a=1,d=8000,s=8,u="115176",},
["132120"]={["?"]=1,d=6000,s=8,t=1000,},
["132365"]={d=20000,s=1,},
["132467"]={["?"]=1,d=100,s=8,},
["134563"]={b="122281",d=-1,s=1,},
["137460"]={d=3000,s=8,},
["137461"]={d=4000,s=8,},
["137562"]={d=6000,s=1,},
["137639"]={d=-1,s=8,},
["138228"]={d=-1,s=8,},
["138233"]={d=1000,s=1,},
["138237"]={d=15000,s=1,},
["138310"]={d=120000,s=1,},
["139597"]={d=15000,s=1,},
["145051"]={d=30000,s=1,},
["147364"]={d=5000,s=1,},
["147494"]={d=60000,s=8,},
["18629"]={s=1,},
["18630"]={s=1,},
["24394"]={["?"]=1,a=1,d=3000,s=1,u="19577",},
["28880"]={["?"]=1,d=15000,s=2,t=3000,},
["32747"]={d=3000,s=1,},
["3276"]={s=1,},
["3277"]={s=1,},
["3278"]={s=1,},
["33697"]={d=15000,s=1,},
["41317"]={s=1,},
["41318"]={s=1,},
["41319"]={s=1,},
["46106"]={s=1,},
["46109"]={s=1,},
["46116"]={s=1,},
["55694"]={d=5000,s=1,t=1000,},
["56481"]={s=1,},
["56486"]={s=1,},
["61481"]={s=1,},
["7928"]={s=1,},
["7929"]={s=1,},
["7934"]={s=1,},
["81722"]={s=1,},
["81724"]={s=1,},
}
end

function SpellInfo.GetClassSpells_rogue()
 return {
["105864"]={d=6000,s=8,},
["108211"]={["?"]=1,b="112961",d=3600000,s=1,},
["108212"]={["?"]=1,b="137573",d=4000,s=1,},
["108215"]={["?"]=1,b="113952",d=3600000,s=1,},
["10840"]={s=1,},
["10841"]={s=1,},
["111240"]={r="1752",s=1,},
["112942"]={d=-1,s=1,},
["112947"]={d=6000,s=1,},
["112961"]={["?"]=1,a=1,d=10000,s=8,u="108211",},
["11327"]={["?"]=1,d=3000,s=1,},
["113746"]={d=30000,s=1,},
["113952"]={["?"]=1,a=1,d=15000,s=8,u="108215",},
["113953"]={d=4000,s=8,},
["114014"]={["?"]=1,s=1,},
["114018"]={["?"]=1,b="115834",d=15000,s=1,t=500,},
["115192"]={d=3000,s=1,},
["115194"]={d=8000,s=8,},
["115196"]={d=12000,s=8,},
["115197"]={d=4000,s=8,},
["115834"]={["?"]=1,a=1,d=1500,s=1,u="114018",},
["119696"]={d=6000,s=1,},
["121153"]={d=10000,s=1,},
["121308"]={d=300000,s=1,},
["121411"]={["?"]=1,s=1,},
["121471"]={["?"]=1,b="121473",d=12000,s=1,},
["121733"]={s=1,},
["122233"]={["?"]=1,d=12000,s=1,t=2000,},
["124271"]={d=60000,s=1,},
["127118"]={s=1,},
["1329"]={b="5374",s=1,},
["1330"]={d=3000,s=1,},
["13750"]={d=15000,s=1,},
["137573"]={["?"]=1,a=1,d=100,s=1,u="108212",},
["137584"]={["?"]=1,e=1,s=1,u="137586",},
["137586"]={["?"]=1,b="137584",d=10000,s=1,},
["137619"]={["?"]=1,b="140149",d=60000,s=1,},
["138151"]={["?"]=1,d=-1,s=1,},
["13877"]={d=-1,s=1,},
["139569"]={s=1,},
["140149"]={["?"]=1,a=1,d=-1,s=1,t=1000,u="137619",},
["14183"]={d=18000,s=1,},
["14185"]={s=1,},
["14189"]={s=1,},
["145193"]={d=30000,s=1,},
["145211"]={d=10000,s=1,},
["145249"]={d=5000,s=1,},
["16511"]={["?"]=1,r="1752",s=1,},
["1725"]={s=1,},
["1752"]={s=1,},
["1766"]={d=5000,s=1,},
["1776"]={d=4000,s=1,},
["1784"]={d=-1,s=1,},
["1804"]={s=1,},
["1833"]={d=4000,s=1,},
["1842"]={s=1,},
["1856"]={["?"]=1,b="18461",s=1,},
["18629"]={s=1,},
["18630"]={s=1,},
["1943"]={d=4000,s=1,t=2000,},
["1966"]={d=5000,s=1,},
["20572"]={d=15000,s=1,},
["2094"]={d=60000,s=1,},
["2098"]={s=1,},
["25046"]={d=2000,s=64,},
["26679"]={d=6000,s=1,},
["2818"]={["?"]=1,a=1,d=12000,s=8,t=3000,u="2823",},
["2823"]={["?"]=1,b="2818",d=3600000,s=1,},
["31224"]={b="35729",d=5000,s=1,},
["32645"]={d=1000,r="2098",s=8,t=1000,},
["32747"]={d=3000,s=1,},
["3276"]={s=1,},
["3277"]={s=1,},
["3278"]={s=1,},
["3408"]={["?"]=1,b="3409",d=3600000,s=1,},
["3409"]={["?"]=1,a=1,d=12000,s=8,u="3408",},
["36554"]={b="36563",d=2000,s=1,},
["408"]={d=1000,s=1,},
["41317"]={s=1,},
["45182"]={d=3000,s=1,},
["46116"]={s=1,},
["51690"]={["?"]=1,d=3000,s=1,t=500,},
["5171"]={d=6000,s=1,t=2000,},
["51713"]={d=8000,s=1,},
["51722"]={d=8000,s=1,},
["51723"]={s=1,},
["5277"]={d=10000,s=1,},
["53"]={s=1,},
["56481"]={s=1,},
["5760"]={["?"]=1,a=1,d=10000,s=8,u="5761",},
["5761"]={["?"]=1,b="5760",d=3600000,s=1,},
["57933"]={["?"]=1,d=6000,s=1,},
["57934"]={["?"]=1,d=20000,s=1,},
["5938"]={s=1,},
["61481"]={s=1,},
["61851"]={["?"]=1,d=3000,s=1,},
["6770"]={d=60000,s=1,},
["68992"]={d=10000,s=1,},
["703"]={b="1330",d=18000,s=1,t=3000,},
["73651"]={s=1,t=3000,},
["73981"]={s=1,},
["74001"]={b="74002",d=20000,s=1,},
["74002"]={d=10000,s=1,},
["76577"]={["?"]=1,d=5000,s=1,t=500,},
["79140"]={d=20000,s=1,},
["7928"]={s=1,},
["7929"]={s=1,},
["7934"]={s=1,},
["81722"]={s=1,},
["84617"]={d=24000,s=1,t=3000,},
["84747"]={d=15000,s=1,},
["8647"]={b="113746",s=1,},
["8676"]={s=1,},
["8679"]={["?"]=1,b="8680",d=3600000,s=1,},
["8680"]={["?"]=1,a=1,d=15000,s=8,u="8679",},
["88611"]={["?"]=1,d=1000,s=1,},
["89775"]={["?"]=1,d=24000,s=1,t=3000,},
["91021"]={d=10000,s=1,},
["93068"]={d=15000,s=1,},
}
end

function SpellInfo.GetClassSpells_mage()
 return {
["10"]={b="42208",d=8000,s=16,t=1000,},
["102051"]={d=8000,s=16,},
["105785"]={["?"]=1,d=30000,s=1,},
["105791"]={["?"]=1,d=30000,s=1,},
["10840"]={s=1,},
["10841"]={s=1,},
["108839"]={d=15000,s=16,},
["108843"]={d=1500,s=4,},
["108853"]={["?"]=1,r="2136",s=4,},
["108978"]={["?"]=1,b="110909",d=10000,s=64,},
["110909"]={["?"]=1,a=1,d=6000,s=64,u="108978",},
["110959"]={["?"]=1,b="110960",s=64,},
["110960"]={["?"]=1,a=1,d=20000,s=64,u="110959",},
["11113"]={d=3000,s=4,},
["111264"]={["?"]=1,b="111340",d=30000,s=16,},
["11129"]={["?"]=1,s=4,},
["111340"]={["?"]=1,a=1,d=5000,s=16,u="111264",},
["112948"]={["?"]=1,d=4000,s=16,t=1000,},
["113092"]={["?"]=1,d=2000,s=16,},
["11366"]={d=18000,s=4,t=3000,},
["113724"]={["?"]=1,b="136511",d=10000,s=16,},
["113862"]={["?"]=1,d=-1,s=1,},
["11426"]={d=60000,s=16,},
["114923"]={["?"]=1,d=12000,s=64,t=1000,},
["114954"]={["?"]=1,s=64,},
["115610"]={d=4000,s=64,},
["115611"]={d=6000,s=64,t=2000,},
["115701"]={d=10000,s=64,},
["115757"]={["?"]=1,d=8000,s=16,},
["115760"]={d=3000,s=16,},
["116"]={["?"]=1,d=15000,s=16,},
["116011"]={["?"]=1,d=60000,s=64,},
["116014"]={["?"]=1,d=-1,s=64,t=1000,},
["116257"]={d=60000,s=1,},
["116267"]={d=25000,s=64,},
["118271"]={d=3000,s=4,},
["118280"]={["?"]=1,s=4,},
["118859"]={["?"]=1,a=1,d=25000,s=64,u="1463",},
["11958"]={s=1,},
["120"]={d=6000,s=16,},
["120145"]={s=64,},
["12042"]={d=15000,s=64,},
["12043"]={d=-1,s=1,},
["12051"]={d=6000,s=64,t=2000,},
["122"]={["?"]=1,d=8000,s=16,},
["12472"]={["?"]=1,d=20000,s=16,},
["125430"]={s=84,},
["126201"]={["?"]=1,s=16,},
["12654"]={d=4000,s=4,t=2000,},
["126578"]={s=1,},
["127117"]={s=1,},
["131078"]={["?"]=1,d=20000,s=16,},
["131079"]={["?"]=1,s=16,},
["131080"]={["?"]=1,s=16,},
["131081"]={["?"]=1,s=20,},
["131581"]={["?"]=1,s=16,},
["132210"]={d=15000,s=1,},
["133"]={["?"]=1,s=4,},
["136511"]={["?"]=1,a=1,d=10000,s=16,t=100,u="113724",},
["138317"]={d=30000,s=64,},
["1449"]={s=64,},
["145252"]={d=10000,s=1,},
["145254"]={d=5000,s=1,},
["145264"]={s=16,},
["1459"]={d=3600000,s=64,},
["1463"]={["?"]=1,b="118859",d=8000,s=64,},
["146557"]={d=15000,s=1,},
["148012"]={["?"]=1,d=30000,s=1,},
["148022"]={["?"]=1,s=16,},
["18629"]={s=1,},
["18630"]={s=1,},
["1953"]={d=1000,s=64,},
["2120"]={d=8000,s=4,t=2000,},
["2136"]={s=4,},
["2139"]={d=6000,s=64,},
["28730"]={d=2000,s=64,},
["28880"]={["?"]=1,d=15000,s=2,t=3000,},
["2948"]={s=4,},
["30451"]={["?"]=1,b="36032",s=64,},
["30455"]={["?"]=1,s=16,},
["30482"]={d=-1,s=4,},
["31589"]={d=15000,s=64,},
["31661"]={d=4000,s=4,},
["31687"]={d=-1,s=16,},
["31707"]={["?"]=1,s=16,},
["32612"]={["?"]=1,d=20000,s=64,},
["32747"]={d=3000,s=1,},
["3276"]={s=1,},
["3277"]={s=1,},
["3278"]={s=1,},
["33395"]={d=8000,s=16,},
["33702"]={d=15000,s=1,},
["36032"]={d=10000,s=64,},
["41320"]={s=1,},
["41425"]={d=30000,s=16,},
["44425"]={s=64,},
["44457"]={d=12000,s=4,t=3000,},
["44544"]={b="30455",d=15000,s=16,},
["44572"]={d=5000,s=16,},
["44614"]={["?"]=1,d=8000,s=20,},
["45438"]={d=10000,s=16,},
["46111"]={s=1,},
["475"]={s=64,},
["48107"]={d=10000,s=4,},
["48108"]={d=15000,s=1,},
["5019"]={s=1,},
["5143"]={["?"]=1,b="7268",d=2000,s=64,t=400,},
["5405"]={s=1,t=1000,},
["55021"]={d=4000,s=64,},
["55342"]={b="58836",d=30000,s=64,t=200,},
["56465"]={s=1,},
["56484"]={s=1,},
["57724"]={d=600000,s=1,},
["57761"]={d=15000,s=16,},
["58836"]={d=30000,s=1,},
["59548"]={["?"]=1,d=15000,s=2,t=3000,},
["59638"]={["?"]=1,d=4000,s=16,},
["6117"]={d=-1,s=64,},
["61316"]={b="61332",d=3600000,s=64,},
["61332"]={d=2000,s=64,},
["66"]={["?"]=1,b="35009",d=3000,s=64,t=1000,},
["68992"]={d=10000,s=1,},
["7268"]={["?"]=1,e=1,s=64,u="5143",},
["7302"]={b="7321",d=-1,s=16,},
["7321"]={d=5000,s=16,},
["759"]={s=64,},
["7928"]={s=1,},
["7929"]={s=1,},
["7934"]={s=1,},
["79683"]={b="5143",d=20000,s=1,},
["80353"]={d=40000,s=64,},
["80354"]={d=600000,s=1,},
["81725"]={s=1,},
["82691"]={["?"]=1,d=10000,s=16,},
["83853"]={["?"]=1,d=10000,s=4,t=1000,},
["84714"]={["?"]=1,d=10000,s=16,},
["84721"]={["?"]=1,d=2000,s=16,},
["87023"]={d=6000,s=4,t=1500,},
["87024"]={d=120000,s=4,},
["88082"]={["?"]=1,s=4,},
["88084"]={["?"]=1,b="36032",s=64,},
}
end

function SpellInfo.GetClassSpells_warlock()
 return {
["103103"]={d=4000,r="686",s=32,t=1000,},
["103958"]={["?"]=1,b="123217",d=-1,s=1,t=1000,},
["103964"]={s=127,},
["103967"]={b="56250",d=1000,s=32,},
["103988"]={b="104314",s=32,},
["104025"]={d=10000,s=4,t=1000,},
["104027"]={s=4,},
["104220"]={["?"]=1,d=10000,s=32,},
["104223"]={["?"]=1,d=30000,s=32,},
["104225"]={["?"]=1,d=300000,s=32,t=2000,},
["104232"]={b="104233",d=8000,s=4,t=1000,},
["104314"]={s=1,},
["104316"]={b="114736",s=1,},
["104318"]={b="104330",s=4,},
["104773"]={d=8000,s=32,},
["105174"]={b="143381",s=36,},
["108359"]={d=12000,s=32,t=1000,},
["108366"]={d=15000,s=32,},
["10840"]={s=1,},
["10841"]={s=1,},
["108416"]={d=20000,s=1,},
["108446"]={d=-1,s=32,},
["108482"]={s=1,},
["108501"]={s=1,},
["108503"]={d=3600000,s=32,t=5000,},
["108508"]={d=10000,s=1,},
["108683"]={d=-1,s=4,},
["108685"]={["?"]=1,d=5000,s=4,},
["109151"]={d=-1,s=32,t=100,},
["109466"]={["?"]=1,d=30000,s=32,},
["109468"]={["?"]=1,d=30000,s=32,},
["109773"]={d=3600000,s=32,},
["1098"]={d=300000,s=32,},
["110913"]={["?"]=1,d=8000,s=1,},
["110914"]={["?"]=1,d=8000,s=32,t=1000,},
["111397"]={["?"]=1,d=60000,s=32,},
["111400"]={d=-1,s=1,t=1000,},
["111685"]={d=60000,s=32,},
["111771"]={["?"]=1,b="113890",d=1,s=32,},
["111859"]={d=20000,s=32,},
["111895"]={d=22000,s=32,},
["111896"]={d=22000,s=32,},
["111897"]={d=22000,s=32,},
["111898"]={d=22000,s=32,},
["1120"]={b="87388",d=12000,s=32,t=2000,},
["112042"]={["?"]=1,d=-1,s=1,},
["112092"]={["?"]=1,b="114328",s=32,},
["1122"]={b="22703",s=32,},
["112866"]={d=-1,s=32,},
["112867"]={d=-1,s=32,},
["112868"]={d=-1,s=32,},
["112869"]={d=-1,s=32,},
["112870"]={d=-1,s=32,},
["112921"]={b="22703",s=32,},
["112922"]={d=60000,s=32,},
["112926"]={d=60000,s=1,},
["112927"]={b="112926",s=32,},
["113858"]={d=20000,r="77801",s=4,},
["113860"]={d=20000,r="77801",s=32,},
["113861"]={d=20000,r="77801",s=32,},
["113886"]={["?"]=1,d=900000,s=32,},
["113890"]={["?"]=1,a=1,d=900000,s=32,u="111771",},
["113901"]={["?"]=1,d=900000,s=32,t=10000,},
["114168"]={d=-1,s=1,t=1000,},
["114189"]={["?"]=1,s=32,},
["114328"]={["?"]=1,b="104314",e=1,s=32,u="112092",},
["114329"]={["?"]=1,s=32,},
["114635"]={s=32,t=1000,},
["114736"]={d=60000,s=32,t=120000,},
["114790"]={d=18000,s=32,t=3000,},
["114925"]={d=-1,s=32,},
["115232"]={b="115234",d=30000,s=32,},
["115422"]={s=36,},
["115625"]={b="104330",d=5000,s=1,},
["115746"]={b="104330",s=4,},
["115748"]={b="104330",s=32,},
["115770"]={d=10000,s=32,},
["115778"]={b="104330",s=32,},
["115781"]={["?"]=1,b="115782",d=6000,s=32,},
["115782"]={["?"]=1,a=1,d=3000,s=32,u="115781",},
["115831"]={b="115832",d=6000,s=1,t=1000,},
["116198"]={["?"]=1,b="116202",d=30000,s=32,},
["116202"]={["?"]=1,b="116198",d=120000,s=32,},
["116858"]={s=32,t=1000,},
["117828"]={d=15000,s=1,},
["118093"]={d=8000,s=1,},
["118699"]={["?"]=1,d=20000,s=32,},
["119652"]={["?"]=1,d=30000,s=32,},
["119653"]={["?"]=1,d=120000,s=32,},
["119678"]={s=32,},
["119899"]={d=12000,s=4,t=3000,},
["120451"]={d=1000,s=4,t=1000,},
["120465"]={s=1,},
["122355"]={b="6353",d=30000,s=4,},
["124915"]={["?"]=1,a=1,d=6000,s=127,u="124916",},
["124916"]={["?"]=1,b="124915",s=127,},
["126"]={d=45000,s=32,},
["127117"]={s=1,},
["131736"]={["?"]=1,s=32,},
["134477"]={["?"]=1,d=-1,s=1,},
["137143"]={["?"]=1,d=4000,s=32,},
["138483"]={["?"]=1,d=20000,s=1,},
["145075"]={d=10000,s=1,},
["145082"]={d=10000,s=1,},
["145085"]={["?"]=1,b="146043",d=10000,s=1,},
["145159"]={d=1500,s=32,},
["145164"]={b="145165",d=5000,s=1,},
["145165"]={d=10000,s=1,},
["1454"]={s=32,},
["146043"]={["?"]=1,a=1,d=10000,s=1,u="145085",},
["146739"]={["?"]=1,a=1,d=18000,s=32,t=2000,u="172",},
["148463"]={d=20000,s=32,},
["1490"]={["?"]=1,d=300000,s=32,t=2000,},
["172"]={["?"]=1,b="146739",s=32,},
["17735"]={d=5000,s=32,},
["17767"]={d=20000,s=32,},
["17877"]={["?"]=1,b="29341",s=32,},
["17962"]={["?"]=1,d=5000,s=4,},
["18223"]={["?"]=1,d=30000,s=32,},
["18540"]={b="60478",s=32,},
["18629"]={s=1,},
["18630"]={s=1,},
["1949"]={b="5857",d=14000,s=4,t=1000,},
["19505"]={s=32,},
["19647"]={b="24259",d=6000,s=32,},
["19658"]={s=32,},
["20310"]={d=2000,s=32,},
["20707"]={b="6203",d=900000,s=32,},
["22703"]={d=2000,s=4,},
["24259"]={d=3000,s=32,},
["27243"]={["?"]=1,d=18000,s=32,t=3000,},
["28730"]={d=2000,s=64,},
["29341"]={["?"]=1,a=1,d=5000,s=32,u="17877",},
["29722"]={r="686",s=4,},
["29858"]={s=32,},
["30108"]={["?"]=1,d=14000,s=32,t=2000,},
["30146"]={d=-1,s=32,},
["30151"]={b="30153",d=6000,s=1,},
["30213"]={b="104330",d=5000,s=1,},
["30283"]={d=3000,s=32,},
["3110"]={b="104330",s=4,},
["31117"]={["?"]=1,d=4000,s=32,},
["32747"]={d=3000,s=1,},
["3276"]={s=1,},
["3277"]={s=1,},
["3278"]={s=1,},
["33702"]={d=15000,s=1,},
["348"]={d=15000,r="172",s=4,t=3000,},
["34936"]={d=8000,s=4,},
["3716"]={b="104330",s=32,},
["41320"]={s=1,},
["46111"]={s=1,},
["47897"]={d=6000,s=32,},
["47960"]={d=6000,s=36,t=1000,},
["48181"]={d=8000,s=32,t=2000,},
["5019"]={s=1,},
["54049"]={b="104330",s=32,},
["54817"]={["?"]=1,d=-1,s=1,},
["5484"]={b="112891",d=20000,s=32,},
["54879"]={["?"]=1,b="120465",d=-1,s=1,},
["56465"]={s=1,},
["56484"]={s=1,},
["5740"]={b="42223",d=6000,s=4,t=1000,},
["5782"]={["?"]=1,s=32,},
["603"]={d=60000,s=32,t=15000,},
["60478"]={d=60000,s=1,},
["60947"]={d=5000,s=32,},
["6201"]={s=32,},
["6229"]={d=30000,s=32,},
["6353"]={b="104314",s=4,},
["6358"]={d=30000,s=32,},
["6360"]={d=10000,s=32,},
["6789"]={b="108396",d=3000,s=32,},
["686"]={["?"]=1,b="104314",s=32,},
["688"]={d=-1,s=32,},
["689"]={["?"]=1,d=6000,s=32,t=1000,},
["68992"]={d=10000,s=1,},
["691"]={d=-1,s=32,},
["697"]={d=-1,s=32,},
["710"]={d=30000,s=32,},
["712"]={d=-1,s=32,},
["74434"]={d=30000,s=32,},
["755"]={["?"]=1,d=6000,s=32,t=1000,},
["77799"]={b="104314",s=36,},
["77801"]={["?"]=1,s=32,},
["7814"]={b="104330",s=32,},
["7870"]={d=300000,s=32,},
["7928"]={s=1,},
["7929"]={s=1,},
["7934"]={s=1,},
["79438"]={d=8000,s=32,},
["80240"]={d=15000,s=32,},
["81725"]={s=1,},
["85692"]={s=32,},
["86121"]={["?"]=1,s=32,},
["86211"]={["?"]=1,d=3000,s=32,},
["87385"]={["?"]=1,s=32,},
["88448"]={d=20000,s=32,},
["89140"]={d=120000,s=1,},
["89420"]={["?"]=1,d=6000,s=32,t=1000,},
["89751"]={b="89753",d=6000,s=1,t=1000,},
["89766"]={d=4000,s=1,},
["89792"]={b="93282",d=200,s=4,},
["89808"]={s=4,},
["93282"]={s=4,},
["980"]={d=24000,s=32,t=2000,},
}
end

function SpellInfo.GetClassSpells_druid()
 return {
["100977"]={d=20000,s=1,},
["102280"]={["?"]=1,b="137452",s=1,},
["102342"]={d=12000,s=8,},
["102351"]={["?"]=1,b="102352",d=30000,s=1,},
["102352"]={["?"]=1,a=1,d=6000,s=8,t=2000,u="102351",},
["102354"]={["?"]=1,d=15000,s=8,},
["102355"]={["?"]=1,d=300000,s=8,},
["102359"]={d=20000,s=8,},
["102401"]={d=500,s=1,},
["102543"]={d=30000,s=1,},
["102546"]={["?"]=1,b="9007",d=3000,s=1,},
["102547"]={["?"]=1,d=-1,s=1,},
["102558"]={d=30000,s=1,},
["102560"]={d=30000,s=1,},
["102703"]={["?"]=1,d=15000,s=8,},
["102706"]={["?"]=1,d=15000,s=8,},
["102791"]={s=8,},
["102793"]={d=10000,s=8,},
["102795"]={d=3000,s=1,t=1000,},
["105713"]={d=15000,s=8,},
["1066"]={d=-1,s=1,},
["106731"]={["?"]=1,d=30000,s=1,},
["106737"]={["?"]=1,d=15000,s=8,},
["106785"]={["?"]=1,s=1,},
["106830"]={["?"]=1,d=15000,s=1,t=3000,},
["106832"]={["?"]=1,s=1,},
["106839"]={["?"]=1,s=1,},
["106898"]={b="97993",d=8000,s=1,},
["106922"]={d=20000,s=1,},
["106951"]={["?"]=1,d=15000,s=1,},
["106952"]={["?"]=1,s=1,},
["106996"]={b="106998",d=10000,s=64,t=1000,},
["1079"]={d=16000,s=1,t=2000,},
["108238"]={s=8,},
["108288"]={["?"]=1,d=45000,s=8,},
["108291"]={["?"]=1,d=45000,s=8,},
["108292"]={["?"]=1,d=45000,s=8,},
["108293"]={["?"]=1,d=45000,s=8,},
["108294"]={["?"]=1,d=45000,s=8,},
["10840"]={s=1,},
["10841"]={s=1,},
["110309"]={d=3600000,s=8,},
["110621"]={b="58836",d=30000,s=64,t=1000,},
["110691"]={["?"]=1,s=8,},
["110807"]={d=30000,s=8,},
["112071"]={b="93431",d=15000,s=1,},
["1126"]={d=3600000,s=8,},
["112997"]={d=10000,s=1,},
["113746"]={d=30000,s=1,},
["113769"]={["?"]=1,s=8,},
["113770"]={["?"]=1,d=30000,s=8,},
["113828"]={["?"]=1,s=8,},
["114108"]={d=6000,s=8,},
["114238"]={d=3000,s=8,},
["115798"]={d=30000,s=1,},
["117679"]={["?"]=1,d=30000,s=1,},
["122114"]={d=-1,s=1,},
["122283"]={s=32,},
["122307"]={["?"]=1,d=6000,s=1,},
["124974"]={d=30000,s=8,},
["127118"]={s=1,},
["127119"]={s=1,},
["127538"]={["?"]=1,d=12000,s=1,t=3000,},
["127663"]={d=4000,s=72,t=1000,},
["127757"]={d=3600000,s=1,},
["132158"]={d=-1,s=1,},
["132365"]={d=20000,s=1,},
["132402"]={["?"]=1,d=6000,s=8,},
["132469"]={["?"]=1,b="61391",s=8,},
["135286"]={["?"]=1,d=10000,s=1,},
["135597"]={["?"]=1,d=15000,s=1,},
["135700"]={["?"]=1,d=15000,s=8,},
["137452"]={["?"]=1,a=1,d=4000,s=1,u="102280",},
["138217"]={d=10000,s=1,},
["138225"]={d=10000,s=32,},
["138351"]={["?"]=1,d=15000,s=8,},
["138358"]={["?"]=1,d=30000,s=8,},
["139569"]={s=1,},
["144770"]={s=64,},
["144772"]={s=8,},
["144865"]={d=6000,s=1,},
["144871"]={d=60000,s=1,},
["144888"]={d=8000,s=1,t=1000,},
["145138"]={d=30000,s=1,},
["145151"]={["?"]=1,d=30000,s=1,},
["145152"]={["?"]=1,d=30000,s=1,},
["145205"]={["?"]=1,d=300000,s=8,},
["145518"]={s=8,},
["146874"]={d=12000,s=1,},
["150017"]={["?"]=1,d=15000,s=1,t=3000,},
["16689"]={b="19975",d=45000,s=8,},
["16870"]={["?"]=1,d=15000,s=8,},
["16886"]={["?"]=1,d=15000,s=8,},
["16914"]={b="42231",d=10000,s=8,t=1000,},
["16949"]={},
["17057"]={d=6000,s=1,},
["1822"]={["?"]=1,d=15000,s=1,t=3000,},
["1850"]={d=15000,s=1,},
["18562"]={s=8,},
["18629"]={s=1,},
["18630"]={s=1,},
["19975"]={["?"]=1,d=27000,s=8,},
["20484"]={s=8,},
["22568"]={s=1,},
["22570"]={s=1,},
["22812"]={b="50411",d=12000,s=8,},
["22842"]={["?"]=1,s=1,},
["24858"]={d=-1,s=1,},
["24907"]={d=-1,s=8,},
["24932"]={d=-1,s=1,},
["2637"]={d=40000,s=8,},
["2782"]={s=64,},
["2908"]={s=8,},
["2912"]={s=64,},
["29166"]={d=10000,s=8,t=1000,},
["3276"]={s=1,},
["3277"]={s=1,},
["3278"]={s=1,},
["33745"]={d=15000,s=1,t=3000,},
["33763"]={["?"]=1,d=15000,s=8,t=1000,},
["33786"]={d=6000,s=8,},
["33878"]={s=1,},
["33891"]={["?"]=1,b="132213",d=-1,s=1,t=7000,},
["339"]={["?"]=1,d=30000,s=8,},
["33943"]={d=-1,s=1,},
["40120"]={d=-1,s=1,},
["41317"]={s=1,},
["41318"]={s=1,},
["41319"]={s=1,},
["44203"]={["?"]=1,a=1,d=8000,s=8,t=2000,u="740",},
["46106"]={s=1,},
["46109"]={s=1,},
["46116"]={s=1,},
["48391"]={d=10000,s=1,},
["48438"]={d=7000,s=8,t=1000,},
["48504"]={d=15000,s=8,},
["48505"]={b="50286",d=10000,s=64,t=1000,},
["48517"]={b="93430",d=-1,s=1,},
["48518"]={b="93431",d=-1,s=1,},
["50322"]={["?"]=1,d=12000,s=1,},
["50334"]={["?"]=1,d=10000,s=1,},
["50411"]={d=3000,s=1,},
["50464"]={s=8,},
["5176"]={["?"]=1,s=8,},
["5185"]={["?"]=1,s=8,},
["5211"]={d=5000,s=1,},
["5215"]={["?"]=1,d=-1,s=1,},
["5217"]={["?"]=1,d=6000,s=1,},
["5221"]={s=1,},
["5229"]={d=10000,s=1,t=1000,},
["52610"]={["?"]=1,d=12000,s=1,t=3000,},
["5487"]={d=-1,s=1,},
["55694"]={d=5000,s=1,t=1000,},
["56481"]={s=1,},
["56486"]={s=1,},
["58180"]={d=12000,s=8,},
["58836"]={d=30000,s=1,},
["61336"]={["?"]=1,d=12000,s=1,},
["61391"]={["?"]=1,a=1,d=6000,s=8,u="132469",},
["61481"]={s=1,},
["62071"]={["?"]=1,d=-1,s=1,},
["62606"]={["?"]=1,s=8,},
["6785"]={s=1,},
["6795"]={d=3000,s=1,},
["6807"]={s=1,},
["68992"]={d=10000,s=1,},
["69369"]={d=8000,s=1,},
["740"]={["?"]=1,b="44203",d=8000,s=8,t=2000,},
["768"]={d=-1,s=1,},
["770"]={d=300000,s=8,},
["774"]={d=12000,s=8,t=3000,},
["77758"]={["?"]=1,d=16000,s=1,t=2000,},
["779"]={["?"]=1,s=1,},
["783"]={d=-1,s=1,},
["78674"]={s=72,},
["78675"]={["?"]=1,b="97547",d=8000,s=8,},
["7928"]={s=1,},
["7929"]={s=1,},
["7934"]={s=1,},
["81098"]={["?"]=1,d=-1,s=1,},
["81192"]={d=3000,s=8,},
["81262"]={d=7000,s=8,t=1000,},
["81281"]={["?"]=1,d=2000,s=8,},
["81283"]={["?"]=1,d=20000,s=8,},
["81722"]={s=1,},
["81724"]={s=1,},
["82365"]={["?"]=1,d=10000,s=1,},
["88423"]={s=64,},
["88747"]={["?"]=1,d=300000,s=8,},
["88751"]={s=8,},
["8921"]={d=14000,s=64,t=2000,},
["8936"]={d=6000,s=8,t=2000,},
["9005"]={["?"]=1,b="9007",d=4000,s=1,},
["9007"]={d=18000,s=1,t=3000,},
["93400"]={d=12000,s=8,},
["93402"]={d=14000,s=8,t=2000,},
["93430"]={d=5000,s=32,},
["93431"]={d=5000,s=32,},
["93622"]={d=3000,s=1,},
["93985"]={["?"]=1,d=4000,s=1,},
["94447"]={["?"]=1,d=15000,s=8,t=1000,},
["96206"]={d=-1,s=8,},
["97547"]={["?"]=1,a=1,d=5000,s=8,u="78675",},
["99"]={d=3000,s=1,},
}
end

function SpellInfo.GetClassSpells_priest()
 return {
["10060"]={d=20000,s=2,},
["105826"]={d=23000,s=8,},
["10840"]={s=1,},
["10841"]={s=1,},
["108920"]={d=500,s=32,},
["108921"]={b="114164",d=10000,s=32,},
["108968"]={["?"]=1,s=32,},
["109964"]={["?"]=1,d=10000,s=2,},
["110744"]={["?"]=1,b="110745",d=15000,s=66,t=250,},
["111758"]={["?"]=1,d=600000,s=2,},
["112833"]={["?"]=1,d=10000,s=1,},
["113792"]={d=30000,s=32,},
["114164"]={d=10000,s=2,},
["114214"]={d=20000,s=2,},
["114239"]={d=5000,s=2,},
["114255"]={b="2061",d=20000,s=1,},
["114404"]={d=20000,s=32,},
["114908"]={["?"]=1,d=15000,s=2,},
["119030"]={["?"]=1,d=6000,s=32,},
["120517"]={["?"]=1,d=5500,s=2,},
["120644"]={["?"]=1,d=5500,s=32,},
["120785"]={["?"]=1,d=100,s=2,},
["121135"]={["?"]=1,s=2,},
["121536"]={d=600000,s=2,},
["122121"]={["?"]=1,b="122128",d=15000,s=96,t=250,},
["122127"]={["?"]=1,s=1,},
["123040"]={b="41967",d=15000,s=32,},
["123254"]={d=10000,s=1,},
["124430"]={d=12000,s=32,},
["124464"]={["?"]=1,s=32,},
["124465"]={["?"]=1,s=32,},
["124495"]={["?"]=1,d=30000,s=32,},
["126135"]={d=180000,s=2,},
["126138"]={b="126137",d=-1,s=2,t=1000,},
["126150"]={d=-1,s=2,},
["126154"]={d=6000,s=2,t=2000,},
["127117"]={s=1,},
["127627"]={["?"]=1,d=100,s=32,},
["127628"]={["?"]=1,d=100,s=32,},
["127632"]={["?"]=1,s=32,},
["129197"]={d=3000,s=32,t=1000,},
["129208"]={d=8000,s=32,},
["129250"]={d=7000,s=2,t=1000,},
["138302"]={s=2,},
["139"]={d=12000,s=2,t=3000,},
["142723"]={["?"]=1,s=32,},
["145180"]={d=12000,s=1,},
["145336"]={d=20000,s=2,},
["145374"]={d=10000,s=2,},
["145724"]={d=1000,s=1,},
["14914"]={d=7000,s=2,t=1000,},
["15286"]={d=15000,s=32,},
["15407"]={d=3000,r="585",s=32,t=1000,},
["15473"]={d=-1,s=32,},
["15487"]={d=5000,s=32,},
["17"]={d=15000,s=2,},
["1706"]={["?"]=1,s=2,},
["18629"]={s=1,},
["18630"]={s=1,},
["19236"]={s=2,},
["2050"]={s=2,},
["2060"]={s=2,},
["2061"]={s=2,},
["2096"]={d=60000,s=32,},
["21562"]={d=3600000,s=2,},
["27827"]={d=15000,s=2,},
["28730"]={d=2000,s=64,},
["28880"]={["?"]=1,d=15000,s=2,t=3000,},
["2944"]={["?"]=1,d=6000,s=32,t=1000,},
["32375"]={b="32592",s=2,},
["32379"]={s=32,},
["32546"]={s=2,},
["32747"]={d=3000,s=1,},
["3276"]={s=1,},
["3277"]={s=1,},
["3278"]={s=1,},
["33076"]={["?"]=1,b="41635",s=2,},
["33206"]={d=8000,s=2,},
["34433"]={b="41967",d=12000,s=32,},
["34861"]={s=2,},
["34914"]={["?"]=1,d=15000,s=32,t=3000,},
["41320"]={s=1,},
["41321"]={s=1,},
["41635"]={["?"]=1,a=1,d=30000,s=2,u="33076",},
["41637"]={["?"]=1,s=1,},
["45242"]={d=8000,s=2,},
["46108"]={s=1,},
["46111"]={s=1,},
["47540"]={["?"]=1,s=2,},
["47585"]={["?"]=1,d=6000,s=32,},
["47666"]={["?"]=1,e=1,s=2,u="47758",},
["47750"]={["?"]=1,s=2,},
["47753"]={d=15000,s=2,},
["47758"]={["?"]=1,b="47666",d=2000,s=2,t=1000,},
["47788"]={d=10000,s=2,},
["48045"]={b="49821",d=5000,s=32,t=1000,},
["5019"]={s=1,},
["527"]={s=2,},
["528"]={s=2,},
["56465"]={s=1,},
["56484"]={s=1,},
["585"]={s=2,},
["586"]={d=10000,s=32,},
["588"]={d=-1,s=2,},
["58880"]={["?"]=1,s=1,},
["589"]={["?"]=1,d=18000,s=32,t=3000,},
["59544"]={["?"]=1,d=15000,s=2,t=3000,},
["596"]={b="98367",s=2,},
["59889"]={d=6000,s=2,},
["60069"]={["?"]=1,d=6000,s=1,t=1000,},
["605"]={d=30000,s=32,},
["62618"]={["?"]=1,d=10000,s=2,},
["6346"]={d=180000,s=2,},
["63619"]={b="35779",d=5000,s=64,},
["63735"]={d=20000,s=2,},
["63853"]={d=12000,s=2,},
["64044"]={["?"]=1,b="64058",d=1000,s=32,},
["64058"]={["?"]=1,a=1,d=8000,s=1,u="64044",},
["64843"]={["?"]=1,b="64844",d=8000,s=2,t=2000,},
["64844"]={["?"]=1,a=1,d=8000,s=2,u="64843",},
["64901"]={["?"]=1,b="64904",d=8000,s=2,t=2000,},
["64904"]={["?"]=1,a=1,d=8000,s=2,u="64901",},
["6788"]={d=15000,s=2,},
["68992"]={d=10000,s=1,},
["73325"]={["?"]=1,b="92572",s=2,},
["73413"]={d=-1,s=2,},
["73510"]={["?"]=1,b="87178",s=48,},
["77489"]={d=6000,s=2,t=1000,},
["77613"]={d=15000,s=2,},
["7928"]={s=1,},
["7929"]={s=1,},
["7934"]={s=1,},
["8092"]={s=32,},
["81206"]={d=-1,s=2,},
["81208"]={d=-1,s=2,},
["81209"]={d=-1,s=2,},
["8122"]={d=8000,s=32,},
["81292"]={d=9000,s=32,},
["81661"]={d=20000,s=1,},
["81700"]={d=18000,s=2,},
["81725"]={s=1,},
["81782"]={["?"]=1,d=500,s=2,},
["87160"]={b="73510",d=10000,s=32,},
["87178"]={["?"]=1,a=1,d=12000,s=48,u="73510",},
["87194"]={d=4000,s=32,},
["88625"]={d=3000,s=2,},
["88684"]={d=6000,s=2,},
["88685"]={d=30000,s=2,t=2000,},
["89485"]={["?"]=1,d=-1,s=1,},
["92572"]={["?"]=1,a=1,d=1500,s=2,u="73325",},
["93400"]={d=12000,s=8,},
["9484"]={d=50000,s=2,},
["95652"]={d=9000,s=2,},
["96267"]={["?"]=1,d=5000,s=1,},
}
end

function SpellInfo.GetClassSpells_hunter()
 return {
["105919"]={["?"]=1,b="108687",d=15000,s=1,},
["10840"]={s=1,},
["10841"]={s=1,},
["108687"]={["?"]=1,a=1,d=15000,s=1,u="105919",},
["109248"]={["?"]=1,d=11000,s=8,t=500,},
["109259"]={s=1,},
["109260"]={d=-1,s=8,},
["109304"]={["?"]=1,b="128594",s=1,},
["1130"]={d=300000,s=64,},
["113746"]={d=30000,s=1,},
["114406"]={["?"]=1,d=5000,s=1,},
["117050"]={["?"]=1,b="120755",d=15000,s=9,t=200,},
["117526"]={["?"]=1,d=5000,s=64,},
["117553"]={["?"]=1,d=10000,s=64,},
["118253"]={["?"]=1,a=1,d=15000,s=8,t=3000,u="1978",},
["118455"]={d=4000,s=1,},
["118694"]={d=-1,s=8,t=2000,},
["119450"]={["?"]=1,d=-1,s=1,},
["120360"]={["?"]=1,b="120361",d=3000,s=1,t=200,},
["120361"]={["?"]=1,e=1,s=1,u="120360",},
["120679"]={d=15000,s=8,},
["120697"]={["?"]=1,d=4000,s=1,t=500,},
["120699"]={["?"]=1,d=15000,s=1,t=3000,},
["120755"]={["?"]=1,e=1,s=1,u="117050",},
["120756"]={["?"]=1,s=1,},
["120761"]={["?"]=1,d=3000,s=1,},
["121414"]={["?"]=1,d=3000,s=1,},
["121818"]={["?"]=1,b="130201",d=20000,s=8,},
["122489"]={["?"]=1,d=15000,s=1,},
["125050"]={s=1,},
["127120"]={s=1,},
["127830"]={d=60000,s=1,},
["128432"]={d=120000,s=1,},
["128433"]={d=120000,s=1,},
["128594"]={["?"]=1,e=1,s=1,u="109304",},
["128997"]={s=1,},
["130201"]={["?"]=1,a=1,d=-1,s=1,u="121818",},
["13159"]={b="15571",d=-1,s=8,},
["13165"]={d=-1,s=8,},
["131894"]={["?"]=1,d=30000,s=1,t=1000,},
["131900"]={["?"]=1,s=1,},
["132950"]={["?"]=1,a=1,d=20000,s=64,u="1543",},
["135299"]={["?"]=1,d=39000,s=16,},
["135678"]={d=-1,s=8,},
["136"]={d=10000,s=8,t=2000,},
["136634"]={d=8000,s=8,},
["13809"]={d=60000,s=16,},
["13810"]={["?"]=1,d=30000,s=16,},
["13812"]={["?"]=1,b="149575",d=20000,s=4,t=2000,},
["13813"]={d=60000,s=4,},
["138363"]={d=10000,s=1,},
["138366"]={s=8,},
["138374"]={s=8,},
["142978"]={d=6000,s=8,},
["144659"]={d=20000,s=1,},
["144670"]={["?"]=1,d=-1,s=1,},
["145737"]={["?"]=1,d=-1,s=1,},
["1462"]={d=30000,s=8,},
["147362"]={d=3000,s=1,},
["1494"]={d=-1,s=1,},
["149575"]={["?"]=1,a=1,d=10000,s=4,u="13812",},
["1499"]={d=60000,s=16,},
["1513"]={d=20000,s=8,},
["1515"]={b="13481",d=10000,s=8,t=10000,},
["1543"]={["?"]=1,b="132950",s=64,},
["15571"]={d=4000,s=1,},
["16827"]={s=1,},
["17253"]={s=1,},
["1742"]={d=6000,s=1,},
["18629"]={s=1,},
["18630"]={s=1,},
["19263"]={["?"]=1,d=5000,s=1,},
["19386"]={d=30000,s=8,},
["19434"]={s=1,},
["19503"]={b="37506",d=4000,s=1,},
["19574"]={d=10000,s=1,},
["19577"]={["?"]=1,b="24394",d=15000,s=8,},
["19615"]={b="82692",d=30000,s=1,},
["1978"]={["?"]=1,b="118253",s=8,},
["19801"]={s=8,},
["19878"]={d=-1,s=1,},
["19879"]={d=-1,s=1,},
["19880"]={d=-1,s=1,},
["19882"]={d=-1,s=1,},
["19883"]={d=-1,s=1,},
["19884"]={d=-1,s=1,},
["20572"]={d=15000,s=1,},
["20736"]={d=6000,s=64,},
["24394"]={["?"]=1,a=1,d=3000,s=1,u="19577",},
["24423"]={d=10000,s=1,},
["24450"]={d=-1,s=1,},
["24529"]={d=-1,s=8,},
["24604"]={d=60000,s=1,},
["24844"]={d=45000,s=8,},
["26090"]={d=2000,s=8,},
["2641"]={b="47531",s=1,},
["2643"]={s=1,},
["2649"]={d=3000,s=1,},
["28880"]={["?"]=1,d=15000,s=2,t=3000,},
["3044"]={s=64,},
["3045"]={d=15000,s=1,},
["32747"]={d=3000,s=1,},
["3276"]={s=1,},
["3277"]={s=1,},
["3278"]={s=1,},
["3355"]={d=60000,s=16,},
["34026"]={s=1,},
["34471"]={d=10000,s=1,},
["34477"]={["?"]=1,d=20000,s=1,},
["34490"]={d=3000,r="147362",s=1,},
["34600"]={d=60000,s=8,},
["34720"]={d=15000,s=8,},
["34889"]={d=45000,s=4,},
["35079"]={["?"]=1,d=4000,s=1,},
["35101"]={d=4000,s=64,},
["35290"]={d=30000,s=1,},
["35346"]={d=6000,s=64,},
["3674"]={d=20000,s=32,t=2000,},
["41314"]={s=1,},
["4167"]={d=5000,s=8,},
["46113"]={s=1,},
["49966"]={s=1,},
["50245"]={d=4000,s=1,},
["50256"]={d=15000,s=1,},
["50274"]={d=10000,s=8,},
["50285"]={s=8,},
["50318"]={d=4000,s=8,},
["50433"]={d=6000,s=1,},
["50479"]={b="62347",s=32,},
["50498"]={b="113746",s=1,},
["50518"]={d=25000,s=1,},
["50519"]={d=2000,s=8,},
["50541"]={d=8000,s=1,},
["5116"]={d=6000,s=1,},
["5118"]={["?"]=1,b="15571",d=-1,s=8,},
["51284"]={s=1,},
["51753"]={["?"]=1,d=60000,s=8,},
["51755"]={["?"]=1,d=60000,s=1,},
["53148"]={["?"]=1,b="57627",d=1000,s=1,},
["53209"]={["?"]=1,s=8,},
["53220"]={d=20000,s=1,},
["53257"]={d=15000,s=1,},
["53271"]={["?"]=1,s=1,},
["53301"]={d=2000,s=4,t=1000,},
["53351"]={["?"]=1,s=1,},
["53353"]={["?"]=1,s=8,},
["53397"]={d=-1,s=1,},
["53479"]={d=20000,s=1,},
["5384"]={d=360000,s=1,},
["54216"]={["?"]=1,d=4000,s=1,},
["54227"]={d=15000,s=1,t=3000,},
["54644"]={b="54689",d=5000,s=24,},
["54680"]={d=10000,s=1,},
["54706"]={d=5000,s=8,},
["55749"]={d=25000,s=8,},
["56453"]={d=12000,s=1,},
["56574"]={s=1,},
["56626"]={d=2000,s=8,},
["56641"]={s=1,},
["57386"]={d=30000,s=1,},
["57627"]={["?"]=1,a=1,d=6000,s=1,u="53148",},
["57724"]={d=600000,s=1,},
["57879"]={d=15000,s=1,},
["58604"]={b="58605",d=10000,s=4,},
["59543"]={["?"]=1,d=15000,s=2,t=3000,},
["61394"]={d=4000,s=1,},
["61482"]={s=1,},
["6197"]={d=60000,s=64,},
["62347"]={d=2000,s=32,},
["63468"]={d=8000,s=1,t=1000,},
["63896"]={d=12000,s=1,},
["64803"]={d=4000,s=8,},
["68992"]={d=10000,s=1,},
["6991"]={b="1539",s=1,},
["75"]={s=1,},
["76663"]={s=1,},
["77767"]={r="56641",s=8,},
["77769"]={d=-1,s=1,},
["781"]={s=1,},
["7928"]={s=1,},
["7929"]={s=1,},
["7934"]={s=1,},
["80483"]={d=2000,s=64,},
["81716"]={s=1,},
["82654"]={d=30000,s=1,},
["82692"]={d=20000,s=1,},
["82726"]={d=10000,s=1,t=1000,},
["82921"]={d=5000,s=1,},
["82925"]={d=30000,s=1,},
["82926"]={d=10000,s=1,},
["82928"]={s=1,},
["83242"]={d=-1,s=1,},
["83243"]={d=-1,s=1,},
["83244"]={d=-1,s=1,},
["83245"]={d=-1,s=1,},
["83559"]={d=39000,s=1,},
["883"]={d=-1,s=1,},
["90309"]={d=60000,s=1,},
["90314"]={b="90315",s=8,},
["90315"]={d=10000,s=8,},
["90327"]={d=4000,s=1,},
["90328"]={d=-1,s=1,},
["90347"]={d=15000,s=8,},
["90355"]={d=40000,s=4,},
["90361"]={d=10000,s=8,t=2000,},
["90364"]={d=-1,s=8,},
["90967"]={["?"]=1,d=6000,s=2,},
["91644"]={d=8000,s=1,},
["92380"]={b="95725",d=8000,s=24,t=2000,},
["93433"]={b="95714",d=8000,s=8,t=1000,},
["93434"]={s=8,},
["93435"]={d=60000,s=1,},
["94019"]={d=12000,s=8,},
["94022"]={s=8,},
["94528"]={["?"]=1,d=20000,s=64,},
["982"]={d=3000,s=8,},
}
end

function SpellInfo.GetClassSpells_deathknight()
 return {
["101568"]={d=15000,s=1,},
["105582"]={d=20000,s=1,},
["105647"]={d=12000,s=1,},
["108194"]={d=5000,s=1,},
["108199"]={d=10000,s=32,},
["108200"]={["?"]=1,b="115000",d=8000,s=16,t=1000,},
["108201"]={["?"]=1,b="118009",d=1,s=32,},
["10840"]={s=1,},
["10841"]={s=1,},
["111673"]={d=300000,s=32,},
["113516"]={d=30000,s=8,},
["114851"]={b="45529",d=25000,s=1,},
["114866"]={["?"]=1,d=5000,s=1,t=5000,},
["114868"]={["?"]=1,d=5000,s=32,},
["115000"]={["?"]=1,a=1,d=3000,s=16,u="108200",},
["115001"]={["?"]=1,d=6000,s=16,},
["115798"]={d=30000,s=1,},
["115989"]={b="115994",d=10000,s=32,t=1000,},
["116783"]={s=32,},
["116888"]={d=3000,s=32,},
["118009"]={["?"]=1,a=1,d=10000,s=32,t=500,u="108201",},
["119975"]={b="119980",d=-1,s=32,t=1000,},
["123693"]={s=1,},
["123981"]={d=180000,s=1,},
["127123"]={s=1,},
["130735"]={["?"]=1,d=5000,s=1,t=5000,},
["130736"]={["?"]=1,d=5000,s=1,t=5000,},
["132365"]={d=20000,s=1,},
["138214"]={s=0,},
["138342"]={d=15000,s=1,},
["144901"]={d=5000,s=1,},
["144909"]={b="147620",d=20000,s=1,},
["144948"]={d=120000,s=1,},
["144953"]={d=10000,s=1,},
["145629"]={["?"]=1,d=-1,s=32,},
["145677"]={d=20000,s=1,},
["18629"]={s=1,},
["18630"]={s=1,},
["20572"]={d=15000,s=1,},
["26064"]={d=12000,s=1,},
["28880"]={["?"]=1,d=15000,s=2,t=3000,},
["32747"]={d=3000,s=1,},
["3276"]={s=1,},
["3277"]={s=1,},
["3278"]={s=1,},
["3714"]={d=600000,s=1,t=250,},
["40274"]={s=1,},
["41312"]={s=1,},
["42650"]={["?"]=1,b="42651",d=4000,s=32,t=500,},
["42651"]={["?"]=1,a=1,d=40000,s=32,u="42650",},
["43265"]={d=10000,s=32,t=1000,},
["45462"]={b="55078",s=1,},
["45477"]={b="55095",s=16,},
["45524"]={["?"]=1,d=8000,s=16,},
["45902"]={s=1,},
["46114"]={s=1,},
["46115"]={s=1,},
["46584"]={["?"]=1,s=1,},
["46585"]={["?"]=1,d=60000,s=1,},
["47468"]={s=1,},
["47476"]={d=5000,s=32,},
["47481"]={["?"]=1,s=1,},
["47482"]={["?"]=1,s=1,},
["47484"]={["?"]=1,s=1,},
["47528"]={d=4000,s=16,},
["47541"]={s=32,},
["47568"]={b="89831",s=1,},
["48263"]={d=-1,s=1,},
["48265"]={d=-1,s=32,},
["48266"]={d=-1,s=16,},
["48707"]={d=5000,s=32,},
["48721"]={s=32,},
["48743"]={s=32,},
["48792"]={d=12000,s=1,},
["48982"]={s=32,},
["49016"]={d=30000,s=1,t=3000,},
["49020"]={s=1,},
["49028"]={["?"]=1,b="1206",d=12000,s=1,},
["49039"]={d=10000,s=32,},
["49143"]={r="45902",s=16,},
["49184"]={s=16,},
["49206"]={["?"]=1,b="61777",d=40000,s=32,t=1000,},
["49222"]={d=300000,s=32,},
["49560"]={["?"]=1,d=3000,s=1,},
["49576"]={["?"]=1,s=32,},
["49998"]={s=1,},
["50421"]={b="49998",d=20000,s=1,},
["50435"]={d=10000,s=16,},
["50452"]={d=20000,s=32,},
["50613"]={d=2000,s=64,},
["50842"]={s=32,},
["51052"]={["?"]=1,d=3000,s=32,},
["51124"]={d=10000,s=1,},
["51271"]={["?"]=1,d=20000,s=1,},
["51460"]={d=3000,s=32,t=3000,},
["51963"]={s=40,},
["53323"]={s=1,},
["53331"]={s=1,},
["53341"]={s=1,},
["53342"]={["?"]=1,s=1,},
["53343"]={s=1,},
["53344"]={s=1,},
["53362"]={["?"]=1,d=-1,s=1,},
["53365"]={d=15000,s=1,},
["53386"]={d=30000,s=16,},
["54446"]={s=1,},
["54447"]={["?"]=1,s=1,},
["54449"]={["?"]=1,d=-1,s=1,},
["55050"]={r="45902",s=1,},
["55078"]={d=30000,s=32,t=3000,},
["55090"]={s=1,},
["55095"]={d=30000,s=16,t=3000,},
["55233"]={d=10000,s=1,},
["55694"]={d=5000,s=1,t=1000,},
["56222"]={d=3000,s=1,},
["56480"]={s=1,},
["56483"]={s=1,},
["56815"]={s=1,},
["57330"]={d=300000,s=1,},
["59052"]={d=15000,s=16,},
["59545"]={["?"]=1,d=15000,s=2,t=3000,},
["61483"]={s=1,},
["61777"]={["?"]=1,a=1,d=30000,s=32,u="49206",},
["61999"]={b="97821",s=32,},
["62158"]={s=1,},
["63560"]={d=30000,s=32,},
["68992"]={d=10000,s=1,},
["70164"]={s=1,},
["73975"]={d=10000,s=1,},
["77535"]={d=10000,s=32,},
["77575"]={b="55078",s=32,},
["77606"]={d=8000,s=1,},
["7928"]={s=1,},
["7929"]={s=1,},
["7934"]={s=1,},
["81141"]={d=15000,s=1,},
["81162"]={["?"]=1,d=8000,s=1,},
["81256"]={["?"]=1,d=12000,s=1,},
["81277"]={d=-1,s=1,},
["81326"]={d=30000,s=1,},
["81340"]={d=10000,s=1,},
["81714"]={s=1,},
["85948"]={s=1,},
["90259"]={["?"]=1,d=20000,s=1,},
["91342"]={b="63560",d=30000,s=1,},
["91797"]={d=4000,s=1,},
["91800"]={["?"]=1,d=3000,s=1,},
["91802"]={["?"]=1,b="91807",s=1,},
["91807"]={["?"]=1,a=1,d=2000,s=1,u="91802",},
["91809"]={["?"]=1,s=1,},
["91837"]={d=10000,s=1,},
["91838"]={["?"]=1,d=10000,s=1,},
["96171"]={["?"]=1,d=8000,s=1,},
["96268"]={d=6000,s=32,},
["96294"]={["?"]=1,d=3000,s=16,},
["97821"]={d=300000,s=1,},
}
end

function SpellInfo.GetUnknown_a()
 return {
abacus_of_violent_odds={"33807",},
abracadaver={"72770",},
absorption={"7848",},
accelerated={"96980","97142",},
accelerated_mending={"33400",},
acceleration={"138703",},
accuracy={"60340",},
adamantine_shell={"33479",},
adepts_elixir={"33740","54452",},
aegis={"67631",},
aegis_heal={"23781",},
aegis_of_dalaran={"71635","71638",},
aegis_of_preservation={"23780",},
agile={"107947","109714","109715","126554",},
agility={"11328",},
agility_of_the_tiger={"102747",},
aim_of_the_iron_dwarves={"71491",},
alacrity={"126657",},
alacrity_of_the_elements={"65004",},
alchemist_stone={"17619",},
alerages_reserve_keg={"127793",},
amazing_fortitude={"92186","92187",},
amberfly_idol={"117646",},
ancestors_vengeance={"16939","144243",},
ancient_pandaren_fishing_charm={"125167",},
ancient_petrified_seed={"97009","97177",},
ancient_power={"35733",},
anthem={"91141","91143",},
arcane_blast={"18091","20883",},
arcane_elixir={"11390","11461",},
arcane_energy={"33662","144073","144108",},
arcane_infused={"23721",},
arcane_infusion={"51987",},
arcane_potency={"24544",},
arcane_secrets={"126588",},
arcane_shroud={"26400",},
arcane_sight={"136089",},
arcanite_dragonling={"19804",},
archers_grace={"136086",},
archivists_emblem={"118612",},
argent_avenger={"17352",},
argent_dawn_banner={"54329","54471",},
argent_glory={"54492",},
argent_heroism={"59658",},
argent_tome_bunny_spawn={"54418",},
argent_valor={"59657",},
argussian_compass={"39228",},
armor={"11349",},
armor_buff={"34199",},
armor_disruption={"36482",},
armor_piercing={"80532",},
armor_shatter={"16928",},
arrow_of_time={"102659",},
ascendance={"28200","28204",},
ashbringer={"28282",},
aura_of_protection={"23506",},
aura_of_the_blue_dragon={"23684",},
aura_of_the_crusader={"39439","39441",},
aura_of_wrath={"39443",},
avoidance={"32600","146343",},
avoidance_of_the_snake={"102741",},
}
end

function SpellInfo.GetUnknown_c()
 return {
caber_toss={"99915",},
call_anathema={"23041",},
call_benediction={"23042",},
call_of_conquest={"84969","92226","99711","99739","102441","105132","126690",},
call_of_dominance={"84968","92225","99712","99741","102437","105134","126683","141376","141377","141969",},
call_of_sulthraze={"11654",},
call_of_the_berserker={"43716",},
call_of_the_nexus={"34321",},
call_of_victory={"84966","92224","99713","99740","102434","105133","126679",},
candied_sweet_potato={"62051","66034",},
cannibalize={"20577","20578",},
cardboard_assassin={"84425","94548",},
carp_hunter_feather={"118613",},
celerity={"67683","91173",},
celestial_celerity={"146296",},
celestial_master={"146312",},
chain_lightning={"16921","21179","144230","144240",},
champion_of_the_dawn={"29112","29113",},
change_of_tactics={"138728",},
chaos_bane={"71904","73422",},
chaos_fire={"24389",},
charged_blows={"101515",},
charm_of_ten_songs={"122687",},
chilled={"16927","20005",},
chitinous_spikes={"26168",},
chromatic_infusion={"27675",},
cleansing_tears={"91138","91139",},
cleave_armor={"15280",},
cloaking={"4079",},
cloudburst={"138856",},
cobalt_frag_bomb={"67890",},
cobra={"75489",},
coin_of_blessings={"120171",},
coin_of_good_fortune={"120174",},
coin_of_luck={"120173",},
coin_of_serendipity={"120172",},
cold_eye={"1139",},
colossus={"116631",},
combat_gallantry={"41263",},
combat_insight={"45041",},
combat_mind={"107970","109793","109795",},
combat_trance={"107960","109717","109719",},
combat_valor={"41261",},
comets_trail={"64772",},
conjure_food={"8736",},
consecrated_weapon={"37360",},
consume_essence={"33012",},
consume_life={"33014",},
coral_adder_medallion={"117644",},
corrosive_poison={"13526","144267",},
corrupted_egg_shell={"96173",},
corruption={"13530","17510","18088","18656",},
covered_in_watermelon={"127723",},
cranberry_chutney={"62049","66035",},
create_belt={"146236","146259","146268","146276",},
create_boots={"146237","146260","146269","146277",},
create_bracer={"146243","146267","146275","146283",},
create_carved_ogre_idol={"69777",},
create_chestpiece={"146238","146261","146270",},
create_cloak={"146246",},
create_curio={"148746",},
create_gloves={"146239","146263","146271","146279",},
create_helm={"146240","146264","146272","146280",},
create_lavalliere={"148740",},
create_leggings={"146241","146265","146273","146281",},
create_perpetual_purple_firework={"69773",},
create_ring={"146244",},
create_robes={"146278",},
create_shoulders={"146242","146266","146274","146282",},
creeping_mold={"18289",},
crescendo_of_suffering={"90996","91002",},
crimson_serpent={"46783",},
cripple={"18381",},
cruel={"146293",},
cruelty={"146285",},
crusaders_glory={"61671",},
cultivated_power={"71570","71572",},
curse_of_stalvan={"13524",},
curse_of_timmy={"17505",},
curse_of_weakness={"8552",},
}
end

function SpellInfo.GetUnknown_b()
 return {
badge_of_kypari_zar={"122695",},
badge_of_the_swarmguard={"26480",},
badger_medallion={"118608",},
bans_bomb={"128365",},
barnacle_crew_despawn_aura={"148596",},
barrens_toughness={"142271",},
basilisk_skin={"10351",},
battle={"91344",},
battle_chicken={"13166",},
battle_magic={"91047",},
battle_prowess={"91374","91376",},
battle_trance={"45040",},
beast_fury={"108016","109860","109863",},
berserk={"59620",},
berserker={"60196",},
berserking={"26297",},
black_diamond_crab={"26609",},
black_magic={"59626","59630",},
black_pearl_panther={"26576",},
blade_warding={"64440",},
blademaster={"92199","92200",},
blades={"138737",},
blades_of_renataki={"138756",},
bladestorm={"9632","35131",},
blast_of_corruption={"107831","109851","109854",},
blaze={"16898",},
blazing_emblem={"13744",},
bleakwood_curse={"16871",},
bleeding_heart={"61620",},
blessing_of_ancient_kings={"64411",},
blessing_of_cenarius={"40452",},
blessing_of_elune={"40446",},
blessing_of_faith={"37877",},
blessing_of_isiset={"91147","91149",},
blessing_of_khazgoroth={"96934","97127",},
blessing_of_life={"38332",},
blessing_of_light={"71870","71872",},
blessing_of_lower_city={"37878",},
blessing_of_remulos={"40445",},
blessing_of_righteousness={"37198",},
blessing_of_the_black_book={"23720",},
blessing_of_the_celestials={"128984","128985","128986","128987",},
blessing_of_the_shaper={"96927",},
blessing_of_the_silver_crescent={"35163",},
blessing_of_zuldazar={"138967",},
blight={"9796",},
blind_spot={"91322","92331",},
blinding_light={"23733",},
blinding_speed={"33489",},
blood_fury={"24571",},
blood_of_power={"138864",},
blood_of_the_old_god={"64790",},
blood_reserve={"64568",},
bloodseekers_fury={"128897",},
bloody_dancing_steel={"142530",},
blossom={"126605",},
bluetip_medallion={"118607",},
bonereavers_edge={"21153",},
bonked={"62991",},
bonus_runic_power={"62458",},
bountiful_drink={"100367",},
bountiful_food={"100365","100374","100376","100379",},
braid_of_ten_songs={"122688",},
brain_damage={"24388",},
brain_hacker={"17148",},
break_tool={"4954",},
breath_of_fire={"12257",},
breath_of_many_minds={"138898",},
brewfest_drink={"44107","44111","44114",},
brilliant_light={"24498",},
brittle_armor={"24574","24575","24590",},
burning_hatred={"32362",},
burst_of_energy={"24532",},
burst_of_knowledge={"15646",},
}
end

function SpellInfo.GetUnknown_e()
 return {
earthen_armor={"79475",},
earthen_guardian={"73550",},
earthshaker={"21152",},
earthstrike={"25891",},
echo_of_archimonde={"21079",},
effervescence={"49623",},
egans_blaster={"17368",},
egg_nog={"21149",},
egg_shell={"91296","91308",},
ejector_seat={"54621",},
electrical_charge={"96890",},
electrified={"43730","138788",},
electromagnetic_pulse={"54735",},
elemental_disruption={"74208",},
elementium_dragonling={"82645",},
elixir_of_accuracy={"60354",},
elixir_of_agility={"11449",},
elixir_of_armor_piercing={"60365",},
elixir_of_brute_force={"17537","17557",},
elixir_of_deadly_strikes={"60355",},
elixir_of_deep_earth={"79480","80488",},
elixir_of_defense={"3177",},
elixir_of_draenic_wisdom={"39627","39638",},
elixir_of_empowerment={"28578",},
elixir_of_expertise={"60357",},
elixir_of_giant_growth={"8240",},
elixir_of_giants={"11472",},
elixir_of_greater_agility={"11467",},
elixir_of_greater_defense={"11450",},
elixir_of_greater_intellect={"11465",},
elixir_of_healing_power={"28545",},
elixir_of_impossible_accuracy={"80491",},
elixir_of_ironskin={"39628","39639",},
elixir_of_lesser_agility={"2333",},
elixir_of_lightning_speed={"60366",},
elixir_of_lions_strength={"2329",},
elixir_of_major_agility={"28553",},
elixir_of_major_defense={"28557",},
elixir_of_major_mageblood={"28570",},
elixir_of_major_strength={"28544",},
elixir_of_mastery={"33726","33741",},
elixir_of_mighty_agility={"53840",},
elixir_of_mighty_defense={"60356",},
elixir_of_mighty_mageblood={"56519",},
elixir_of_mighty_speed={"80493",},
elixir_of_mighty_strength={"54218",},
elixir_of_minor_accuracy={"63732",},
elixir_of_minor_agility={"3230",},
elixir_of_minor_defense={"7183",},
elixir_of_mirrors={"105687","114763",},
elixir_of_ogres_strength={"3188",},
elixir_of_peace={"105685","114764",},
elixir_of_perfection={"105686","114762",},
elixir_of_protection={"54220",},
elixir_of_spirit={"53747","53847",},
elixir_of_superior_defense={"17554",},
elixir_of_the_cobra={"79477","80484",},
elixir_of_the_giants={"11405",},
elixir_of_the_master={"79635","80497",},
elixir_of_the_mongoose={"17538","17571",},
elixir_of_the_naga={"79474","80480",},
elixir_of_the_rapids={"105684","114759",},
elixir_of_the_sages={"17535","17555",},
elixir_of_weaponry={"105683","114756",},
elixir_of_wisdom={"3171",},
elusive={"107951","109778","109779","126533",},
elusive_power={"67669","71579",},
embedded_spear={"137658",},
embers_of_azzinoth={"40393",},
emblem_of_kypari_zar={"122693",},
emblem_of_the_catacombs={"122311",},
embrace_of_the_spider={"60492",},
emerald_boar={"56188",},
emerald_owl={"26600","61368",},
emissarys_watch={"95880",},
emp_generator={"54736",},
empowerment={"134870",},
empyrean_tortoise={"46780",},
enchant_2h_weapon__agility={"27837",},
enchant_2h_weapon__greater_impact={"13937",},
enchant_2h_weapon__greater_savagery={"44630",},
enchant_2h_weapon__impact={"13695",},
enchant_2h_weapon__lesser_impact={"13529",},
enchant_2h_weapon__lesser_intellect={"7793",},
enchant_2h_weapon__lesser_spirit={"13380",},
enchant_2h_weapon__major_agility={"27977",},
enchant_2h_weapon__major_intellect={"20036",},
enchant_2h_weapon__major_spirit={"20035",},
enchant_2h_weapon__massacre={"60691",},
enchant_2h_weapon__mighty_agility={"95471",},
enchant_2h_weapon__minor_impact={"7745",},
enchant_2h_weapon__savagery={"27971",},
enchant_2h_weapon__scourgebane={"44595",},
enchant_2h_weapon__superior_impact={"20030",},
enchant_boots__agility={"13935",},
enchant_boots__assassins_step={"74252",},
enchant_boots__assault={"60606",},
enchant_boots__blurred_speed={"104409",},
enchant_boots__boars_speed={"34008",},
enchant_boots__cats_swiftness={"34007",},
enchant_boots__dexterity={"27951",},
enchant_boots__earthen_vitality={"74189",},
enchant_boots__fortitude={"27950",},
enchant_boots__greater_agility={"20023",},
enchant_boots__greater_assault={"60763",},
enchant_boots__greater_fortitude={"44528",},
enchant_boots__greater_haste={"104407",},
enchant_boots__greater_precision={"104408",},
enchant_boots__greater_spirit={"44508",},
enchant_boots__greater_stamina={"20020",},
enchant_boots__greater_vitality={"44584",},
enchant_boots__haste={"74199",},
enchant_boots__icewalker={"60623",},
enchant_boots__lavawalker={"74253",},
enchant_boots__lesser_accuracy={"63746",},
enchant_boots__lesser_agility={"13637",},
enchant_boots__lesser_spirit={"13687",},
enchant_boots__lesser_stamina={"13644",},
enchant_boots__major_agility={"74213",},
enchant_boots__mastery={"74238",},
enchant_boots__minor_agility={"7867",},
enchant_boots__minor_speed={"13890",},
enchant_boots__minor_stamina={"7863",},
enchant_boots__pandarens_step={"104414",},
enchant_boots__precision={"74236",},
enchant_boots__spirit={"20024",},
enchant_boots__stamina={"13836",},
enchant_boots__superior_agility={"44589",},
enchant_boots__surefooted={"27954",},
enchant_boots__tuskarrs_vitality={"47901",},
enchant_boots__vitality={"27948",},
enchant_bracer__agility={"96264",},
enchant_bracer__assault={"60616",},
enchant_bracer__brawn={"27899",},
enchant_bracer__critical_strike={"74201",},
enchant_bracer__dodge={"13931",},
enchant_bracer__exceptional_intellect={"44555",},
enchant_bracer__exceptional_spirit={"74237",},
enchant_bracer__exceptional_strength={"104390",},
enchant_bracer__expertise={"44598",},
enchant_bracer__fortitude={"27914",},
enchant_bracer__greater_agility={"104391",},
enchant_bracer__greater_assault={"44575",},
enchant_bracer__greater_critical_strike={"74248",},
enchant_bracer__greater_dodge={"27906",},
enchant_bracer__greater_expertise={"74239",},
enchant_bracer__greater_intellect={"20008",},
enchant_bracer__greater_speed={"74256",},
enchant_bracer__greater_spellpower={"44635",},
enchant_bracer__greater_spirit={"13846",},
enchant_bracer__greater_stamina={"13945",},
enchant_bracer__greater_stats={"44616",},
enchant_bracer__greater_strength={"13939",},
enchant_bracer__healing_power={"23802",},
enchant_bracer__intellect={"13822",},
enchant_bracer__lesser_assault={"34002",},
enchant_bracer__lesser_dodge={"13646",},
enchant_bracer__lesser_intellect={"13622",},
enchant_bracer__lesser_spirit={"7859",},
enchant_bracer__lesser_stamina={"13501",},
enchant_bracer__lesser_strength={"13536",},
enchant_bracer__major_dodge={"104385",},
enchant_bracer__major_intellect={"34001",},
enchant_bracer__major_spirit={"44593",},
enchant_bracer__major_stamina={"62256",},
enchant_bracer__major_strength={"96261",},
enchant_bracer__mana_regeneration={"23801",},
enchant_bracer__mastery={"104338",},
enchant_bracer__mighty_intellect={"96262",},
enchant_bracer__minor_agility={"7779",},
enchant_bracer__minor_dodge={"7428",},
enchant_bracer__minor_health={"7418",},
enchant_bracer__minor_spirit={"7766",},
enchant_bracer__minor_stamina={"7457",},
enchant_bracer__minor_strength={"7782",},
enchant_bracer__precision={"74232",},
enchant_bracer__restore_mana_prime={"27913",},
enchant_bracer__speed={"74193",},
enchant_bracer__spellpower={"27917",},
enchant_bracer__spirit={"13642",},
enchant_bracer__stamina={"13648",},
enchant_bracer__stats={"27905",},
enchant_bracer__strength={"13661",},
enchant_bracer__super_intellect={"104389",},
enchant_bracer__superior_dodge={"74229",},
enchant_bracer__superior_healing={"27911",},
enchant_bracer__superior_spellpower={"60767",},
enchant_bracer__superior_spirit={"20009",},
enchant_bracer__superior_stamina={"20011",},
enchant_bracer__superior_strength={"20010",},
enchant_chest__dodge={"46594",},
enchant_chest__exceptional_health={"27957",},
enchant_chest__exceptional_mana={"27958",},
enchant_chest__exceptional_resilience={"44588",},
enchant_chest__exceptional_spirit={"74231",},
enchant_chest__exceptional_stats={"27960",},
enchant_chest__glorious_stats={"104395",},
enchant_chest__greater_dodge={"47766",},
enchant_chest__greater_health={"13640",},
enchant_chest__greater_mana={"13663",},
enchant_chest__greater_mana_restoration={"44509",},
enchant_chest__greater_stamina={"74251",},
enchant_chest__greater_stats={"20025",},
enchant_chest__health={"7857",},
enchant_chest__lesser_absorption={"13538",},
enchant_chest__lesser_health={"7748",},
enchant_chest__lesser_mana={"7776",},
enchant_chest__lesser_stats={"13700",},
enchant_chest__major_health={"20026",},
enchant_chest__major_mana={"20028",},
enchant_chest__major_resilience={"33992",},
enchant_chest__major_spirit={"33990",},
enchant_chest__mana={"13607",},
enchant_chest__mighty_health={"44492",},
enchant_chest__mighty_resilience={"74214",},
enchant_chest__mighty_spirit={"104393",},
enchant_chest__mighty_stats={"74191",},
enchant_chest__minor_absorption={"7426",},
enchant_chest__minor_health={"7420",},
enchant_chest__minor_mana={"7443",},
enchant_chest__minor_stats={"13626",},
enchant_chest__peerless_stats={"74250",},
enchant_chest__powerful_stats={"60692",},
enchant_chest__restore_mana_prime={"33991",},
enchant_chest__stamina={"74200",},
enchant_chest__stats={"13941",},
enchant_chest__super_health={"47900",},
enchant_chest__super_resilience={"104392",},
enchant_chest__super_stats={"44623",},
enchant_chest__superior_health={"13858",},
enchant_chest__superior_mana={"13917",},
enchant_chest__superior_stamina={"104397",},
enchant_cloak__accuracy={"104398",},
enchant_cloak__critical_strike={"74230",},
enchant_cloak__defense={"13635",},
enchant_cloak__dodge={"25086",},
enchant_cloak__greater_agility={"34004",},
enchant_cloak__greater_critical_strike={"74247",},
enchant_cloak__greater_defense={"13746",},
enchant_cloak__greater_dodge={"47051",},
enchant_cloak__greater_intellect={"74240",},
enchant_cloak__greater_protection={"104401",},
enchant_cloak__greater_speed={"47898",},
enchant_cloak__intellect={"74202",},
enchant_cloak__lesser_agility={"13882",},
enchant_cloak__lesser_power={"74192",},
enchant_cloak__lesser_protection={"13421",},
enchant_cloak__major_agility={"60663",},
enchant_cloak__major_armor={"27961",},
enchant_cloak__mighty_stamina={"47672",},
enchant_cloak__minor_agility={"13419",},
enchant_cloak__minor_power={"44582",},
enchant_cloak__minor_protection={"7771",},
enchant_cloak__protection={"74234",},
enchant_cloak__pvp_power={"34003",},
enchant_cloak__shadow_armor={"44631",},
enchant_cloak__speed={"60609",},
enchant_cloak__stealth={"25083",},
enchant_cloak__subtlety={"25084",},
enchant_cloak__superior_agility={"44500",},
enchant_cloak__superior_critical_strike={"104404",},
enchant_cloak__superior_defense={"20015",},
enchant_cloak__superior_dodge={"44591",},
enchant_cloak__superior_intellect={"104403",},
enchant_cloak__wisdom={"47899",},
enchant_gloves__advanced_herbalism={"13868",},
enchant_gloves__advanced_mining={"13841",},
enchant_gloves__agility={"13815",},
enchant_gloves__angler={"71692",},
enchant_gloves__armsman={"44625",},
enchant_gloves__assault={"33996",},
enchant_gloves__blasting={"33993",},
enchant_gloves__crusher={"60668",},
enchant_gloves__exceptional_spellpower={"44592",},
enchant_gloves__exceptional_strength={"74212",},
enchant_gloves__expertise={"44484",},
enchant_gloves__fire_power={"25078",},
enchant_gloves__fishing={"13620",},
enchant_gloves__frost_power={"25074",},
enchant_gloves__gatherer={"44506",},
enchant_gloves__greater_agility={"20012",},
enchant_gloves__greater_assault={"44513",},
enchant_gloves__greater_blasting={"44612",},
enchant_gloves__greater_expertise={"74220",},
enchant_gloves__greater_haste={"104416",},
enchant_gloves__greater_mastery={"74255",},
enchant_gloves__greater_strength={"20013",},
enchant_gloves__haste={"74198",},
enchant_gloves__healing_power={"25079",},
enchant_gloves__herbalism={"13617",},
enchant_gloves__major_agility={"44529",},
enchant_gloves__major_healing={"33999",},
enchant_gloves__major_spellpower={"33997",},
enchant_gloves__major_strength={"33995",},
enchant_gloves__mastery={"74132",},
enchant_gloves__mighty_strength={"74254",},
enchant_gloves__mining={"13612",},
enchant_gloves__minor_haste={"13948",},
enchant_gloves__precise_strikes={"33994",},
enchant_gloves__precision={"44488",},
enchant_gloves__riding_skill={"13947",},
enchant_gloves__shadow_power={"25073",},
enchant_gloves__skinning={"13698",},
enchant_gloves__strength={"13887",},
enchant_gloves__super_strength={"104419",},
enchant_gloves__superior_agility={"25080",},
enchant_gloves__superior_expertise={"104417",},
enchant_gloves__superior_mastery={"104420",},
enchant_gloves__threat={"25072",},
enchant_offhand__major_intellect={"104445",},
enchant_offhand__superior_intellect={"74235",},
enchant_ring__agility={"74216",},
enchant_ring__assault={"44645",},
enchant_ring__greater_agility={"103461",},
enchant_ring__greater_intellect={"103462",},
enchant_ring__greater_stamina={"103463",},
enchant_ring__greater_strength={"103465",},
enchant_ring__intellect={"74217",},
enchant_ring__lesser_intellect={"44636",},
enchant_ring__lesser_stamina={"59636",},
enchant_ring__minor_intellect={"27924",},
enchant_ring__stamina={"74218",},
enchant_ring__stats={"27927",},
enchant_ring__strength={"74215",},
enchant_ring__striking={"27920",},
enchant_shield__dodge={"44489",},
enchant_shield__greater_intellect={"60653",},
enchant_shield__greater_parry={"130758",},
enchant_shield__greater_spirit={"13905",},
enchant_shield__greater_stamina={"20017",},
enchant_shield__intellect={"27945",},
enchant_shield__lesser_dodge={"27944",},
enchant_shield__lesser_parry={"13689",},
enchant_shield__lesser_protection={"13464",},
enchant_shield__lesser_spirit={"13485",},
enchant_shield__lesser_stamina={"13631",},
enchant_shield__major_stamina={"34009",},
enchant_shield__mastery={"74226",},
enchant_shield__minor_stamina={"13378",},
enchant_shield__parry={"27946",},
enchant_shield__protection={"74207",},
enchant_shield__resilience={"44383",},
enchant_shield__spirit={"13659",},
enchant_shield__stamina={"13817",},
enchant_shield__vitality={"20016",},
enchant_staff__greater_spellpower={"62948",},
enchant_staff__spellpower={"62959",},
enchant_weapon__accuracy={"59619",},
enchant_weapon__agility={"23800",},
enchant_weapon__avalanche={"74197",},
enchant_weapon__battlemaster={"28004",},
enchant_weapon__berserking={"59621",},
enchant_weapon__black_magic={"59625",},
enchant_weapon__blade_ward={"64441",},
enchant_weapon__blood_draining={"64579",},
enchant_weapon__colossus={"104440",},
enchant_weapon__crusader={"20034",},
enchant_weapon__dancing_steel={"104434",},
enchant_weapon__deathfrost={"46578",},
enchant_weapon__demonslaying={"13915",},
enchant_weapon__elemental_force={"104430",},
enchant_weapon__elemental_slayer={"74211",},
enchant_weapon__exceptional_agility={"44633",},
enchant_weapon__exceptional_spellpower={"44629",},
enchant_weapon__exceptional_spirit={"44510",},
enchant_weapon__executioner={"42974",},
enchant_weapon__fiery_weapon={"13898",},
enchant_weapon__giant_slayer={"44621",},
enchant_weapon__greater_agility={"42620",},
enchant_weapon__greater_potency={"60621",},
enchant_weapon__greater_striking={"13943",},
enchant_weapon__healing_power={"22750",},
enchant_weapon__heartsong={"74225",},
enchant_weapon__hurricane={"74223",},
enchant_weapon__icebreaker={"44524",},
enchant_weapon__icy_chill={"20029",},
enchant_weapon__jade_spirit={"104427",},
enchant_weapon__landslide={"74246",},
enchant_weapon__lesser_beastslayer={"13653",},
enchant_weapon__lesser_elemental_slayer={"13655",},
enchant_weapon__lesser_striking={"13503",},
enchant_weapon__lifestealing={"20032",},
enchant_weapon__lifeward={"44576",},
enchant_weapon__major_healing={"34010",},
enchant_weapon__major_intellect={"27968",},
enchant_weapon__major_spellpower={"27975",},
enchant_weapon__major_striking={"27967",},
enchant_weapon__mending={"74195",},
enchant_weapon__mighty_intellect={"23804",},
enchant_weapon__mighty_spellpower={"60714",},
enchant_weapon__mighty_spirit={"23803",},
enchant_weapon__minor_beastslayer={"7786",},
enchant_weapon__minor_striking={"7788",},
enchant_weapon__mongoose={"27984",},
enchant_weapon__potency={"27972",},
enchant_weapon__power_torrent={"74242",},
enchant_weapon__rivers_song={"104442",},
enchant_weapon__soulfrost={"27982",},
enchant_weapon__spellpower={"22749",},
enchant_weapon__spellsurge={"28003",},
enchant_weapon__strength={"23799",},
enchant_weapon__striking={"13693",},
enchant_weapon__sunfire={"27981",},
enchant_weapon__superior_potency={"60707",},
enchant_weapon__superior_striking={"20031",},
enchant_weapon__titanguard={"62257",},
enchant_weapon__unholy_weapon={"20033",},
enchant_weapon__windsong={"104425",},
enchant_weapon__windwalk={"74244",},
enchant_weapon__winters_might={"21931",},
enchanted_lantern={"93841",},
endless_blessings={"34210",},
endurance_of_niuzao={"148010",},
enduring_judgment={"40472",},
enduring_light={"40471",},
energized={"67696","67750",},
energized_shield={"24499",},
energy_siphon={"65008",},
engulfing_shadows={"27860",},
enigma={"92123",},
enjoying_a_cold_one={"101582",},
enlightened={"43722",},
enlightenment={"29601","35095","126266","141158",},
enrage={"12686",},
ephemeral_power={"23271",},
escalating_power={"67726","67740",},
escalation={"67723","67739",},
escape_artist={"20589",},
eskhandars_rage={"22640",},
eskhandars_rake={"22639",},
essence_flow={"60527",},
essence_infused_mushroom={"33758",},
essence_of_gossamer={"60218",},
essence_of_life={"60062","101289","127915",},
essence_of_sapphiron={"28779",},
essence_of_the_eternal_flame={"97010","97179",},
essence_of_the_martyr={"35165",},
essence_of_yulon={"146198","148954",},
evasive_maneuvers={"45058",},
every_man_for_himself={"59752",},
evocation={"45052",},
executioner={"42976",},
expanded_mind={"146046",},
expansive_mind={"107962","109811","109813",},
expansive_soul={"91155",},
expertise={"60344",},
exploit_weakness={"40461",},
explosive_barrel={"127858",},
exquisite_proficiency={"133630",},
extract_of_necromatic_power={"60488",},
extravagant_visions={"148897",},
eye_of_brutality={"139170",},
eye_of_divinity={"23101",},
eye_of_doom={"91368","91370",},
eye_of_the_broodmother={"65006",},
eye_of_vengeance={"92094","92096",},
eyes_of_twilight={"75490","75495",},
}
end

function SpellInfo.GetUnknown_d()
 return {
damage_absorb={"23991","25746","25747","25750",},
dancing_steel={"120032",},
dangle_wild_carrot={"49266",},
dark_iron_luck={"51952",},
dark_iron_pipeweed={"51953",},
dark_iron_scorpid={"26614",},
darkglow={"125488",},
darkglow_embroidery={"55769","75175","125482",},
dawnstone_crab={"31039",},
dazed={"13496",},
dead_winds={"90953","90985",},
deadeye={"136088",},
deadly_aim={"40487",},
deadly_precision={"71563","71564",},
deadly_strikes={"60341",},
death_by_peasant={"18307","18308","93742",},
death_by_voodoo_gnome={"43995","98275",},
deathfrost={"46629",},
decayed_strength={"13528",},
deep_meditation={"40402",},
deep_twilight_serpent={"68351",},
defender_of_the_timbermaw={"26066",},
defenders_code={"60286",},
defense={"3220",},
defensive_maneuvers={"146344",},
defensive_tactics={"67694",},
deflection={"52419",},
delusional={"105225",},
demon_panther={"73549",},
demonfork={"16603",},
destiny={"17152",},
destiny_fulfilled={"38284",},
determination={"146250",},
detonate_mana={"92601",},
devilsaur_fury={"24352",},
devour={"96911","97125",},
devourers_stomach={"95879",},
dextrous={"146308",},
diabolic_remedy={"43710",},
diamond_flask={"24427",},
dietary_enhancement={"91338",},
dire_drunkard={"51955",},
dire_magic={"91007","92318",},
disarm={"11879","13534","15752",},
disdain={"45053",},
dispel_magic={"16908",},
dispel_poison={"21954",},
displacement={"38351",},
divine_blessing={"40440",},
divine_wrath={"40441",},
dnd_cancel_shapeshift_and_mounts={"105011",},
draconic_leg_reinforcements={"124561","124566","124569",},
dragons_call={"13049",},
dragonspine_flurry={"34775",},
dream_owl={"73552",},
drink={"1137","27089","43706","57073","66041","72623","87958","87959","104262","104269","104270","105230","130335","130336","130337","130338","130339","130340","130341",},
druid_of_the_flames={"99245",},
drunken_evasiveness={"68443","101293","127967",},
duelist={"92205","92208",},
dying_breath={"93229",},
dying_curse={"60494",},
}
end

function SpellInfo.GetUnknown_g()
 return {
gear_detected={"92055",},
ghost_elixir={"79468","80477",},
giant_growth={"8212",},
giant_wave={"89182",},
gift_of_life={"23725","23782",},
gift_of_stone={"16470",},
glade_pincher_feather={"118614",},
glade_singer_medallion={"118610",},
gleaming={"127569",},
glyph_of_deflection={"84212",},
gnome_ingenuity={"84213",},
gnomish_battle_chicken={"23133",},
gnomish_death_ray={"13278",},
gnomish_lightning_generator={"55039",},
goblin_bomb={"23134",},
goblin_dragon_gun={"13183",},
goblin_glider={"126389","126392",},
goblin_mortar={"13237",},
going_ape={"48332","48333",},
golden_dream_badge={"122921",},
golden_dream_emblem={"122919",},
golden_dream_insignia={"122920",},
golden_dream_relic={"122917",},
golden_dream_sigil={"122918",},
golden_fleece_effect_2={"129750",},
golden_fleece_effect_3={"129751",},
golden_hare={"26571",},
golden_moss_effect_2={"148556",},
golden_moss_effect_3={"148555",},
golems_strength={"79634",},
grace={"92085","92089",},
grace_of_earth={"25892",},
great_fortitude={"92172",},
greater_agility={"11334",},
greater_arcane_elixir={"17539","17573",},
greater_armor={"11348",},
greater_intellect={"11396",},
greater_mana_regeneration={"28509",},
greater_potion_of_luck={"105699",},
greater_rune_of_warding={"42137",},
greater_stoneshield={"17540",},
greenpaw_idol={"117649",},
grim_toll={"60437",},
grounded_plasma_shield={"82626","82627","84427",},
grounded_soul={"91184","92332",},
grove_viper_medallion={"117643",},
grummlecake={"127843",},
grunts_tenacity={"134953",},
guardian_effect={"10342",},
gurus_elixir={"53749","53848",},
gutgore_ripper={"21151","69180","69181",},
}
end

function SpellInfo.GetUnknown_f()
 return {
fade={"12685",},
faded_forest_badge={"118754",},
faded_forest_emblem={"118751",},
faded_forest_insignia={"118753",},
faded_forest_medal={"118752",},
faded_forest_medallion={"118750",},
faerie_fire={"13752",},
fang_of_the_crystal_spider={"17331",},
farseeing_eyes={"51985",},
fatal_flaws={"71403",},
fatality={"91355",},
favored={"91345",},
fearwurm_badge={"122690",},
fearwurm_relic={"122686",},
feathers_of_fury={"138759",},
fecundity={"38333",},
fel_curse={"12938",},
fel_immolation={"109907",},
fel_infusion={"40396",},
fel_strength_elixir={"38954","38960",},
felfire_inscription={"86403",},
felsteel_boar={"31038",},
felstriker={"16551",},
feral_fury={"48848",},
ferocity={"33667","144074","148896",},
fervor={"806",},
fiery_quintessence={"97008","97176",},
figurine__monarch_crab={"59757",},
filling={"41920",},
final_key={"92069","92091",},
final_word={"127577",},
find_weakness={"107988","109742","109744",},
fire_blood={"40459",},
fire_reflector={"23097",},
fireball={"13438","15662","16413","16415","18082","18199","18796","21159","21162",},
firebolt={"13442","18083","18086","18112","18833","23267","144247","144270",},
flame_of_the_heavens={"64713",},
flame_wrath={"16560",},
flamelager_kegwell_despawn_aura={"128529",},
flamelager_medallion={"117645",},
flamelagers_summer_brew={"127788",},
flamelagers_summer_keg={"127789",},
flashfreeze={"126478",},
flashing_steel={"126484",},
flask_of_falling_leaves={"105693","114772",},
flexweave_underlay={"55002",},
flintlockes_woodchucker={"99621",},
flurry_of_xuen={"146194","148957","149276",},
foam_sword_attack={"62973",},
foaming_rage={"47217",},
focus={"18803","90900",},
focused_mind={"31794",},
focused_power={"32355",},
food={"5004","5005","5006","5007","10256","18230","24800","24869","25660","32112","33253","33255","33258","33260","33262","33264","33266","33269","35271","40768","41030","43763","43777","45548","46683","46898","53283","64056","65418","65419","65420","65421","65422","87544","87566","87567","87568","87570","87571","87572","87573","87577","87584","87586","87587","87588","87594","87595","87597","87599","87601","87602","87604","87636","87637","104235","104934","104935","105231","108029","108030","108033","108034","130327","130328","130329","130330","130332","130333","130334","148726",},
footmans_resolve={"134944",},
force_of_will={"15595",},
forceful_strike={"40477",},
forethought_talisman={"60530",},
forge_ember={"60479",},
forged_documents={"86654","89244",},
forged_fury={"91836",},
forgotten_knowledge={"38317",},
fortitude={"67699","67753","137593",},
foul_gift={"102662","109908",},
fountain_of_light={"71864","71866",},
frag_belt={"54793",},
frenzy={"138895",},
frenzyheart_fury={"59821",},
frightalon={"19755",},
frost_blast={"16407","18092","18204","18276","18398","19260","20869","144254",},
frost_reflector={"23131",},
frost_vulnerability={"51714",},
frostbolt={"13439","96089","144249","144251",},
frostmourne={"43827",},
frostscythe={"46643",},
fur_lining__agility={"57683","85008","124551",},
fur_lining__intellect={"57691","85010","124552",},
fur_lining__stamina={"57690","85007","124553",},
fur_lining__strength={"85009","124549","124554",},
furnace_stone={"65011",},
fury={"67671",},
fury_of_forgewright={"15494",},
fury_of_the_beast={"108011","109861","109864",},
fury_of_the_crashing_waves={"42084",},
fury_of_the_destroyer={"109949",},
fury_of_the_earthen={"90889",},
fury_of_the_five_flights={"60314",},
}
end

function SpellInfo.GetUnknown_i()
 return {
icy_rage={"71401","71541",},
illdari_bane={"37649",},
illidans_fury={"22988",},
illusionary_barrier={"57350",},
illustration_of_the_dragon_soul={"60486",},
image_of_immortality={"92222",},
imbibe={"105222",},
impale={"34580",},
implosion={"65024",},
impossible_accuracy={"79481",},
incendiary_fireworks={"109076",},
incendiary_fireworks_launcher={"109077",},
incisor_fragment={"60299",},
increased_agility={"18192",},
increased_fortitude={"71569",},
increased_spirit={"18193",},
increased_stamina={"25661",},
indestructible={"53762",},
indomitable={"108008",},
infernal_protection={"36483",},
infiltrators_guile={"62088",},
infinite_power={"61428","139189",},
infinite_speed={"61427",},
infinite_spirit={"61426",},
infuse_armor__belt={"141843",},
infuse_armor__boots={"141863",},
infuse_armor__breastplate={"141861",},
infuse_armor__gloves={"141864",},
infuse_armor__helm={"141865",},
infuse_armor__leggings={"141866",},
infuse_armor__shoulders={"141867",},
inner_brilliance={"126577","142157",},
inner_eye={"91320","92329",},
insane_strength_potion={"28494",},
inscription_of_the_earth_prince={"86402",},
insight_of_the_ashtongue={"40483",},
insight_of_the_qiraji={"26481",},
insignia_of_kypari_zar={"122694",},
intellect_of_the_sage={"102748",},
invigorated={"71575","71577","92043",},
invigoration={"71881","71882","71883","71886","71887","71888",},
invisibility_field={"82820","84348","84424",},
inward_contemplation={"146323",},
ironshield={"28515",},
item__purify={"14134",},
}
end

function SpellInfo.GetUnknown_h()
 return {
handmounted_pyro_rocket={"54998",},
hardened={"67727","67741",},
hardened_shell={"92166","92174","144203",},
hardened_skin={"43713","71586",},
hardening_armor={"67728","67742",},
haste={"13494","13533","21165","28507","144233","144234","144238",},
haste_of_the_mongoose={"102744",},
headless_horseman_laugh={"43873",},
headmasters_charge={"18264",},
heal={"14053",},
healing_focus={"47807",},
healing_of_the_ages={"24998",},
healing_power={"28491","36347",},
healing_purity={"48855",},
healing_trance={"37706","60515",},
healthy={"111840",},
heart_of_a_dragon={"60305",},
heart_of_fire={"126260",},
heart_of_iron={"64763",},
heart_of_the_scale={"17275",},
heartened={"91363","91364",},
heartrazor={"36041",},
hearts_judgment={"91041","92328",},
hearts_mystique={"33486",},
hearts_revelation={"91027","92325",},
heartsong={"74224",},
heartsparked={"95875",},
heartwarmer={"136087",},
heavy_leg_reinforcements={"124563","124565","124568",},
heavy_lifting={"91336",},
heedless_carnage={"92108",},
heightened_reflexes={"40729",},
herald_of_doom={"92052","92087","144201",},
heroism={"32955","39200",},
holiday_drink={"45020",},
holy_energy={"45062",},
holy_shield={"9800",},
holy_strength={"20007",},
hopped_up={"51954",},
horn_of_the_traitor={"93248",},
hospitality={"67684",},
howling_blade={"13490",},
hunger={"129812",},
hurricane={"74221",},
hymn_of_power={"90989","90992",},
hyperspeed_acceleration={"54758",},
hyperspeed_accelerators={"54999",},
}
end

function SpellInfo.GetUnknown_k()
 return {
karma={"127572",},
khorium_boar={"46782",},
kidnapped_puppies_despawn_aura={"130119",},
king_of_boars={"73522",},
kiss_of_death={"90895",},
kiss_of_the_spider={"28866",},
knightly_valor={"136091",},
knockdown={"15753",},
knot_of_ten_songs={"122689",},
korkron_elite={"134954",},
}
end

function SpellInfo.GetUnknown_j()
 return {
jade_owl={"26551","61365",},
jade_raccoon_despawn_aura__hw={"122732",},
jade_spirit={"104993",},
jade_warlord_figurine={"126597",},
jadefied={"125410",},
jangthraze={"11657",},
jeweled_serpent={"73551",},
jom_gabbar={"29602","29604",},
jormungar_slime={"51978",},
jousters_fury={"63250",},
juju_madness={"138938",},
julies_blessing={"8348",},
jungle_huntress_idol={"118615",},
}
end

function SpellInfo.GetUnknown_m()
 return {
mad_hozen_elixir={"105682","114754",},
mageblood_elixir={"24363","24365",},
magic_disruption={"36478",},
magic_lamp={"93843",},
magic_weapon_dnd={"121992",},
magistrike={"109888",},
maizer_leaf={"118867",},
majestic_dragon_figurine={"60525",},
major_agility={"54494",},
major_armor={"28502",},
major_strength={"28490",},
malowns_slam={"17500",},
mana_infusion={"28760",},
mana_spring_totem={"24854",},
mana_surge={"37445",},
manipulators_wrath={"128853",},
mantid_elixir={"105681","114755",},
mantid_poison={"128386",},
mark_of_norgannon={"60319",},
mark_of_salvation={"148908",},
mark_of_the_catacombs={"122309",},
mark_of_the_chosen={"21970",},
mark_of_the_firelord={"97007","97146",},
mark_of_the_war_prisoner={"60480",},
marlis_brain_boost={"24268",},
martar_despawn_aura={"130108",},
massive_destruction={"24543",},
master_pit_fighter={"109993",},
master_tactician={"92188","107986","109774","109776",},
master_whisper_aura={"100785",},
master_whisper_aura_final={"101151",},
master_whisper_aura_ii={"100873",},
mastermind={"139133",},
masters_inscription_of_the_axe={"61117",},
masters_inscription_of_the_crag={"61118",},
masters_inscription_of_the_pinnacle={"61119",},
masters_inscription_of_the_storm={"61120",},
masters_spellthread={"56034","75154","125496",},
mastery_of_nimbleness={"102742",},
matrix_restabilized={"96977","97139",},
mauler_medallion={"118609",},
mechanical_dragonling={"4073",},
mechanohog={"60866",},
medallion_of_heroism={"60986",},
medallion_of_the_catacombs={"122312",},
meditation={"38346",},
mekgineers_chopper={"60867",},
memories_of_love={"65003",},
memory_of_invincibility={"92213","92357",},
menders_charm={"136090",},
mental_protection_field={"36480",},
mentally_prepared={"92162",},
mercurial_shield={"26463","26464","26465",},
metamorphosis_rune={"23724",},
meteoric_inspiration={"64999","65000",},
meteorite_whetstone={"60302",},
mighty={"138760",},
mighty_agility={"28497",},
mighty_defense={"60343",},
mighty_earthquake={"89181",},
mighty_mana_regeneration={"53764",},
mighty_rage={"17528",},
mighty_speed={"79632",},
mighty_strength={"53748",},
mind_amplification_dish={"67799","67839",},
mind_flay={"52586",},
mind_quickening={"23723",},
mindfletcher={"90842",},
minor_absorption={"7423",},
minor_accuracy={"63729",},
minor_agility={"2374",},
minor_defense={"673",},
minor_mount_speed={"13927",},
mirror_scope={"109092",},
mirror_strider_emblem={"117648",},
mist_incarnation_medallion={"118606",},
misty_jade_idol={"117651",},
mithril_insignia={"12733",},
mithril_mechanical_dragonling={"12749",},
mithril_spurs={"7215",},
mjolnir_runestone={"65019",},
moderate_insight={"84746",},
mogu_rune_of_paralysis={"129554",},
mojo_madness={"43712",},
monks_elixir={"105688","114758",},
monster_slayers_kit={"54092",},
mote_of_anger={"71432",},
mothallus_spinneret={"128524",},
mount_speed={"48776","48777",},
mountainscaler_badge={"120258",},
mountainscaler_emblem={"120256",},
mountainscaler_insignia={"120257",},
mountainscaler_mark={"120254",},
mountainscaler_medal={"120255",},
multistrike={"146071",},
munificence={"127549",},
mushan_horn={"118869",},
mystical_disjunction={"25768",},
}
end

function SpellInfo.GetUnknown_l()
 return {
lacerations={"10370",},
lalas_song={"95877",},
lance_equipped={"62853",},
landslide={"74245",},
lavanthors_talisman={"60215",},
lead_plating={"92179","92184",},
learning={"483",},
lesser_absorption={"7447",},
lesser_agility={"3160",},
lesser_rune_of_warding={"42134",},
lesser_shielding={"29674",},
lesser_warding_shield={"29503",},
lessons_of_the_darkmaster={"126519",},
leviathan={"91135","91136",},
lifeblood={"138724",},
lifestone_healing={"17712",},
lightning_bolt={"37661","54843","96891",},
lightning_in_a_bottle={"95870",},
lightning_speed={"28093","60346",},
lightning_strike={"137597",},
lights_embrace={"95216",},
lightweave={"125487",},
lightweave_embroidery={"55642","75172","125481",},
limitless_power={"45044",},
linkens_boomerang={"15712",},
lionheart={"34513","144232",},
lions_strength={"2367",},
lionsmane_inscription={"86401",},
living_flame={"64712",},
living_ice_crystals={"60526",},
living_ruby_serpent={"31040",},
loathebs_reflection={"28778",},
loathebs_shadow={"60439",},
longfang_tooth={"118870",},
loom_of_fate={"96945","97129",},
lord_blastingtons_scope_of_doom={"109085",},
lord_generals_sword={"15602",},
lorewalkers_emblem={"120177",},
lorewalkers_insignia={"120180",},
lorewalkers_mark={"120176",},
lorewalkers_medallion={"120179",},
lorewalkers_sigil={"120178",},
lucidity={"137247","137288","137323","137326","137331",},
lucky_rabbits_foot={"118873",},
lucky_springtail_foot={"118866",},
luckydo_coin={"120175",},
luffa={"23595",},
lunks_kudos={"93749",},
lust_for_battle={"35166",},
}
end

function SpellInfo.GetUnknown_o()
 return {
obsidian_insight={"26166",},
ogre_rage={"75488",},
ogres_strength={"3164",},
omarions_gift={"95881",},
on_guard={"62972",},
oozing_wound={"54697",},
oracle_ablutions={"59789",},
orange_painted_turnip={"127821",},
ordon_death_chime={"148534","148535","148536",},
outrage={"146245",},
}
end

function SpellInfo.GetUnknown_n()
 return {
nature_aligned={"23734",},
necrotic_touch={"71875","71877",},
needle_and_thread={"136083",},
nefarious_plot={"92124","92349",},
netherwing_ally={"40811","40815",},
netomatic={"13099","13120",},
nightmare={"109955",},
nightseye_panther={"31047",},
nimble={"92071",},
nimble_fingers={"50261",},
nimble_healing_touch={"24542",},
nitro_boosts={"54861","55004","55016",},
no_feather_fall={"79636",},
now_is_the_time={"60064","101291","127923",},
numbing_pain={"16528",},
nurongs_gun_blast={"128191",},
}
end

function SpellInfo.GetUnknown_q()
 return {
quickened_tongues={"126659","142159",},
quickflip_deflection_plates={"82176","82177",},
quickness_of_the_sailor={"50263",},
quilen_statuette={"130486",},
quilen_statuette_despawn_aura={"130484",},
}
end

function SpellInfo.GetUnknown_p()
 return {
pagles_broken_reel={"24610",},
painted_turnip={"127819","127820","128521",},
pandamonium={"110761",},
pandaren_dragonling={"131411",},
pandoras_plea={"64741",},
paragon={"67703","67772",},
paralyzed={"129553",},
pattern_of_light={"91192",},
perceived_weakness={"37174",},
perfect_aim={"138963",},
perfection={"127575",},
perfumed_grace={"48865",},
persistent_shield={"26467",},
persuasive_strike={"52781",},
phalanx={"36372",},
phantom_strike={"9806",},
phase_fingers={"108788","108789",},
physical_protection={"21956",},
piccolo_of_the_flaming_fire={"18400",},
piercing_twilight={"75456","75458",},
pillar_of_flame={"67714","67760",},
pit_fighter={"109994",},
plainshawk_feather={"118872",},
platinum_disks_of_battle={"64524",},
platinum_disks_of_sorcery={"64525",},
platinum_disks_of_swiftness={"64527",},
poised_to_strike={"126513",},
poison={"8313","13518","16400","16401","17330","17511","18077","18197","18203","18208","21952","26610","144248","144265",},
poison_cloud={"11790","12766","93740",},
polarization={"91351","91352",},
potion_of_brawlers_cunning={"134987",},
potion_of_brawlers_deftness={"134989",},
potion_of_brawlers_might={"134986",},
potion_of_heroes={"28506",},
potion_of_luck={"135855",},
potion_of_mogu_power={"105706",},
potion_of_the_jade_serpent={"105702",},
potion_of_the_mountains={"105698",},
poultryized={"30501",},
poultryizer={"30507",},
power_circle={"45042","45043",},
power_infused_mushroom={"33743",},
power_of_focus={"92045",},
power_of_prayer={"32367",},
power_of_the_ashtongue={"40480",},
power_of_the_guardian={"28142","28143","28144","28145",},
power_of_the_sun_king={"36070",},
power_surge={"40466",},
power_torrent={"74241",},
prayer_beads_blessing={"24354",},
precise_strikes={"48847",},
predation={"126476",},
preternatural_evasion={"107968","109780","109782",},
primal_instinct={"50708",},
primal_leg_reinforcements={"124559","124564","124567",},
prismatic={"90847",},
prismatic_elixir={"80492","134873",},
prismstone={"19638",},
projectile_vomit={"78830",},
protection={"53763",},
protection_of_the_celestials={"128988",},
protectors_vigor={"40464",},
pummel={"13491",},
pumped_up={"100322",},
pumpkin_pie={"62044","66036",},
pumpkin_soldier={"50062",},
puncture_armor={"11791","17315","144260",},
pure_awesome={"58783",},
pure_genuis={"75487",},
purified_spirit={"54839",},
pvp_trinket={"42292",},
pyrite_infusion={"65014",},
pyro_rocket={"54757",},
}
end

function SpellInfo.GetUnknown_s()
 return {
sanctified_orb={"24865",},
sanctified_spellthread={"56039","75155","125497",},
sanctuary={"22850","69207","69208",},
santos_blessing={"38293",},
sapphire_owl={"56186","56187",},
scale_of_fates={"64707",},
scaly_nimbleness={"75477","75480",},
scavengers_badge={"116724",},
scavengers_emblem={"116721",},
scavengers_insignia={"116723",},
scavengers_medal={"116722",},
scavengers_medallion={"116720",},
scorpions_lethality={"142286",},
scroll_of_revered_ancestors={"126606","141162",},
scything_talons={"61778",},
seal_of_the_dawn={"23930",},
seal_of_the_pantheon={"60214",},
searing_blast={"16454",},
searing_words={"95874",},
seaspray_albatross={"46785","48983",},
second_wind={"15604",},
seeping_willow={"17196",},
sense_for_weakness={"136084",},
shadopan_dragon_gun={"129115","129116",},
shadow_bolt={"13440","13480","14106","14122","16408","16409","16783","16784","17483","18138","18205","18211","18214","18217","45055","69211","69212","144246","144250","144252","144257","144258","144268","144271","144272",},
shadow_fox_tail={"118868",},
shadow_reflector={"23132",},
shadowbolt_volley={"108005","109798","109800",},
shadowfiend={"132602",},
shadowmeld={"58984",},
shadows_fate={"71169",},
shadows_of_the_destroyer={"109941",},
shadowsong_panther={"46784",},
shahram={"16602",},
shallow_insight={"84745",},
shard_of_the_fallen_star={"26789",},
shell_of_deterrence={"31771",},
shield_of_hydra_sputum={"140380",},
shoot_plague={"43333",},
shoots_of_life={"117650",},
shot_power={"37508",},
shovel={"89089",},
show_of_faith={"64739",},
shrink_ray={"13003","13006",},
si7_training={"134945",},
sigil_of_compassion={"122314",},
sigil_of_devotion={"122318",},
sigil_of_fidelity={"122315",},
sigil_of_grace={"122316",},
sigil_of_kypari_zar={"122692",},
sigil_of_patience={"122317",},
sigil_of_the_catacombs={"122310",},
silence={"18278",},
silkbead_emblem={"117647",},
silkspawn_carving={"118611",},
silkspawn_wing={"118871",},
singing_cricket_medallion={"117642",},
siphon_essence={"40291","40293","101140","101146",},
siphon_health={"18652",},
siphoned_power={"71605","71636",},
six_demon_bag={"14537",},
skittering_badge={"122900",},
skittering_emblem={"122898",},
skittering_insignia={"122899",},
skittering_relic={"122896",},
skittering_sigil={"122897",},
skullforge_brand={"17484",},
skycallers_swiftness={"48868",},
skyfire_swiftness={"39959",},
skyflare_swiftness={"55379",},
slayer={"91810",},
slayers_crest={"28777",},
slippery={"126236",},
slowing_the_sands={"107804","109842","109844",},
slowroasted_turkey={"62045","66037",},
smite_demon={"13907",},
smokeys_lighter={"17283",},
smouldering={"101093",},
snap_root_tuber={"133024",},
socks={"121717",},
socks_absorb={"128534",},
solarians_grace={"58157",},
sonic_awareness={"55018",},
sonic_shield={"55019",},
soothe={"32599","35352",},
soothing_power={"148911",},
soul_barrier={"138979",},
soul_breaker={"17506",},
soul_fragment={"71905","96962","97131",},
soul_power={"91019",},
spark_of_life={"60520",},
spark_of_zandalar={"138958",},
speaking_of_rage={"107821","109856","109858",},
speed={"53908",},
speed_infusion={"36479",},
speed_of_thought={"92098","92099",},
spell_haste={"33370",},
spell_power={"32956","35337","36432","39201","47816","48875","144130",},
spell_vulnerability={"23605",},
spice_bread_stuffing={"62050","66038",},
spinal_healing_injector={"82184","82200",},
spirit_of_chiji={"146200","148956",},
spirit_of_wisdom={"102746",},
spotlight={"25823",},
sprung={"61815",},
squirmy_delight={"127882",},
squirmy_feeling={"127881",},
star_of_light={"54739",},
static_charge={"136082",},
static_electricity={"3742",},
stay_of_execution={"96988","97145",},
stay_withdrawn={"96993",},
stoicism={"32957",},
stoneform={"20594","65116",},
stoneshield={"4941",},
strength={"60229",},
strength_of_courage={"102740",},
strength_of_stone={"12731",},
strength_of_the_champion={"16916","144239",},
strength_of_the_taunka={"71561",},
stun={"56","17308","23454","34510",},
stunning_blow={"15283",},
suffering={"109959",},
sulthraze={"11658",},
summon_argent_knight={"54308",},
summon_crew_of_the_barnacle={"148597",},
summon_crimson_cannon={"6251",},
summon_essence_of_storms={"137883",},
summon_fallen_footman={"92336",},
summon_fallen_grunt={"92337",},
summon_goblin_nurse={"78922",},
summon_image_of_wrathion={"140195",},
summon_martar={"127464",},
summon_moonwell={"100612","100623",},
summon_pumpkin_soldier_missile={"50071",},
summon_pumpkin_soldiers={"50070",},
summon_random_vanquished_tentacle={"64981",},
summon_reclaimed_thunderstrike={"68787",},
summon_reinforced_thunderstrike={"69419",},
summon_shadowstrike={"21181",},
summon_smithing_hammer={"11209",},
summon_splashing_waters={"101492",},
summon_splashing_waters_visual={"101495",},
summon_tentacle_of_the_old_ones={"107818","109838","109840",},
summon_terracotta_warrior={"127247",},
summon_the_black_brewmaiden={"48042","68271","101285",},
summon_the_brewmaiden={"48041","68270","101286",},
summon_thunderstrike={"21180",},
summon_tracking_hound={"9515",},
summon_tranquil_sprout={"130146",},
summon_tuskarr_fishing_spear={"128329",},
summon_valkyr={"71843","71844",},
summon_voidwalker={"25112",},
summon_whirlwind_of_blades={"127265",},
sunwell_torrent={"28516",},
superior_durability={"136092",},
superluminal={"138699",},
supremacy_of_the_alliance={"134946",},
supremacy_of_the_horde={"134956",},
surge_of_conquest={"85022","92216","92220","99717","99748","102439","105135","126707",},
surge_of_dominance={"85027","92218","99719","99742","102435","105137","126705",},
surge_of_power={"71601","71644",},
surge_of_strength={"17499","138702",},
surge_of_victory={"85032","99721","99746","102432","105139","126700",},
surging_power={"71600","71643",},
swarmkeepers_speed={"128882","128887",},
swiftsteel_inscription={"86375",},
swordguard_embroidery={"55777","75178","125483","125489",},
symbol_of_the_catacombs={"122313",},
synapse_springs={"82174","82175","96228","96230","126731","126734","141331",},
}
end

function SpellInfo.GetUnknown_r()
 return {
race_against_death={"91821","92342",},
radiance={"126640",},
rage={"67695",},
rage_of_the_fallen={"71396",},
rage_of_the_unraveller={"33649",},
rageheart={"91816","92345",},
ragged_johns_neverending_cup={"20587",},
rainbow_generator={"95224",},
rampage={"138870",},
ramsteins_lightning_bolts={"17668","17669",},
rapid_healing={"24546",},
raw_fury={"91832",},
razor_frost={"50401",},
readiness={"145955","146019","146025",},
recombobulate={"18805","23064",},
recurring_power={"34747",},
reflection_of_torment={"60065","101287","127928",},
refocus={"24531",},
refreshing_drink={"111841",},
refreshing_food={"111842",},
refreshment={"57085","57096","57098","57101","57106","57110","57138","57285","57287","57289","57324","57326","57328","57331","57333","57335","57341","57343","57344","57354","57355","57357","57359","57362","57364","57366","57370","57372","58067","104263","104266","104284","104285","104286","104287","104288","104289","104290","104291","104292","104293","104294","104295","104296","124239","124240","124241","124242","124243","124244","124245","124246","124247","124248","124249","124250","125073","125074","125107","125109","125114","125116","130359","130360","130361","130362","130363","130364","130365","130366","130367","130368","130369","130370","130371","130372","131829","142273","142281","142287","145266","145267","145300","145301","145302","145303","148720","148727","148744","148745","148747","148749",},
regeneration={"38324","38325",},
release_of_light={"71607","71646",},
release_puppies={"127233",},
relentlessness={"126489",},
relic_of_kypari_zar={"122691",},
rend={"13318","14118","16403","16406","17153","17504","18075","18078","18200","18202","21949","144263","144264",},
reorigination={"139120",},
replenish_mana={"33394","71565","71574",},
resolute={"60180",},
restless_agility={"146310",},
restless_spirit={"146317",},
restless_strength={"24661","24662",},
retaliation={"52424",},
reticulated_armor_webbing={"63770","82387",},
reveal_the_shadows={"109954",},
revelation={"91024","92320",},
revitalized={"71584",},
rheas_child={"95185",},
risen_fury={"67737","67746",},
rising_fury={"67738","67747",},
river_of_death={"92104",},
rivers_song={"116660",},
rocket_barrage={"69041",},
rocket_jump={"69070",},
rocket_launch={"46567",},
rosary_of_light={"102660","110008",},
royal_seal_of_king_llane={"65012",},
ruby_hare={"56121",},
ruby_serpent={"26599",},
rune_of_repulsion={"60258",},
rune_of_the_dawn={"24198",},
runic_infusion={"47215","48846",},
}
end

function SpellInfo.GetUnknown_u()
 return {
ultimate_power={"107948","109791","109792",},
underlords_mandible={"95882",},
undying_flames={"95872",},
unholy_curse={"20006",},
universal_remote={"8344",},
unleash_tornado={"127282",},
unlock_armor_cache={"147597","148099","148103","148104",},
unrelenting_attacks={"126649",},
unstable_currents={"38348",},
unstable_power={"24658","24659",},
untamed_fury={"23719",},
untouchable={"126646",},
unwavering_might={"126582",},
unyielding_courage={"34106",},
unyielding_knights={"38162",},
urgency={"71568",},
ursine_blessing={"37340",},
use_filled_brewfest_stein={"41943","41944","41945","41946",},
uthers_light_effect={"10368",},
}
end

function SpellInfo.GetUnknown_t()
 return {
tactician={"146395",},
talasite_owl={"31045","48986",},
talisman_of_the_alliance={"33828",},
talisman_of_the_horde={"32140",},
talisman_of_troll_divinity={"60517",},
tawnyhide_antler={"118875",},
tazik_shocker={"82179","82180",},
tears_of_anguish={"58904",},
tectonic_shift={"92233",},
temporal_rift={"35353",},
tempus_repit={"137590",},
tenacious={"148899",},
tenacity={"33668","40538","45049",},
tendon_rip={"44622",},
tendrils_of_darkness={"90896","90898",},
tentacle_call={"26391",},
tentacles={"61619",},
terracotta_warrior_despawn_aura={"130067",},
the_arcanists_stone={"34000",},
the_blades_song={"38282",},
the_breaking={"22989",},
the_breaking_left_blade_dnd={"22991",},
the_burrowers_shell={"29506",},
the_dark_of_night={"38307",},
the_darkest_night={"59043",},
the_decapitator={"37208",},
the_deepest_night={"95678","127890",},
the_eye_of_diminution={"28862",},
the_eye_of_the_dead={"28780",},
the_forming={"22990",},
the_generals_heart={"64765",},
the_lion_horn_of_stormwind={"18946",},
thick_skin={"71633","71639",},
thrill_of_victory={"91828",},
throw_spear={"137660",},
thunder_clap={"13532",},
thunderfury={"21992","27648",},
tidal_charm={"835",},
tilt_at_windmills={"93225",},
times_favor={"34519",},
tipping_of_the_scales={"96880",},
titanic_power={"96923",},
titanic_restoration={"146314",},
titanic_strength={"107949","107966","109746","109747","109748","109750",},
tolvir_agility={"79633",},
tome_of_arcane_phenomena={"60471",},
tosselwrenchs_shrinker={"95227",},
touched_by_a_troll={"60518",},
towering_rage={"47806",},
toxic_power={"148906",},
tranquil_sprout_despawn_aura={"130156",},
tremendous_fortitude={"44055","55915","67596","84960","92223","99714","99737","102438","105144","126697","137211","144129",},
truesilver_boar={"26593",},
truesilver_crab={"26581",},
turn_of_the_worm={"92235","92355",},
twilight_firelance_equipped={"74180",},
twilight_flames={"75466","75473",},
twilight_renewal={"75493","75494",},
twilight_serpent={"56184",},
twisted={"92126","92351",},
typhoon={"91340","91341","144205",},
}
end

function SpellInfo.GetUnknown_7()
 return { }
end

function SpellInfo.GetUnknown_6()
 return { }
end

function SpellInfo.GetUnknown_9()
 return { }
end

function SpellInfo.GetUnknown_8()
 return { }
end

function SpellInfo.GetUnknown_z()
 return {
zandalari_warding={"138925",},
zandalari_warrior={"138960",},
zeal={"8191",},
}
end

function SpellInfo.GetUnknown_y()
 return {
yaungol_charge={"126558",},
yaungol_fire={"126211",},
yaungol_wind_chime={"126555",},
yaungol_wind_chime_cancel={"126565",},
yulons_bite={"146218",},
}
end

function SpellInfo.GetUnknown_x()
 return {
xray_targeting={"95712",},
}
end

function SpellInfo.GetUnknown_w()
 return {
war_stomp={"20549",},
warm_glow={"61617",},
wasteland_badge={"122270",},
wasteland_emblem={"122268",},
wasteland_insignia={"122269",},
wasteland_relic={"122266",},
wasteland_sigil={"122267",},
watergliding_jets={"109099",},
watermelon_bomb={"127721","128519",},
weakening_disease={"18633",},
weapon_equip_timed_instruction_aura={"128680",},
webbed={"127763",},
well_fed={"19705","19706","19708","19709","19710","24799","33254","33256","33257","33259","33261","33263","33265","33268","35272","42293","43764","44097","44100","44104","44105","45245","46687","46899","53284","57079","57097","57100","57102","57107","57111","57139","57286","57288","57291","57294","57325","57327","57329","57332","57334","57356","57358","57360","57365","57367","57371","64057","65410","65415","87545","87546","87547","87548","87549","87550","87551","87552","87554","87555","87556","87557","87558","87559","87560","87561","87562","87563","87564","87634","87635","100368","100373","100375","100377","104264","104267","104271","104272","104273","104274","104275","104276","104277","104278","104279","104280","104281","104282","104283","105226","108028","108031","108032","110645","124210","124211","124212","124213","124214","124215","124216","124217","124218","124219","124220","124221","125070","125071","125106","125108","125113","125115","130342","130343","130344","130345","130346","130347","130348","130350","130351","130352","130353","130354","130355","130356","131828","146804","146805","146806","146807","146808","146809","147312",},
whack_summon_aura={"101994",},
whackagnoll={"101612",},
whirlwind_of_blades_despawn_aura={"130070",},
whisper_of_spirits={"127570",},
white_ash={"148388",},
wild_magic={"53909",},
will_of_the_forsaken={"7744",},
windreaper={"20586",},
winds_of_time={"148447",},
windsong={"104509",},
windswept_pages={"126483",},
windwalk={"74243",},
winged_talisman={"60521",},
wisdom={"3166","37656",},
witching_hour={"90885","90887",},
world_breaker={"36111",},
worldqueller={"90927",},
wound={"17407","21961",},
wracking_pains={"54696",},
wrath={"18104","146202",},
wrath_of_the_darkspear={"146184",},
wrathstone={"64800",},
wushoolays_lightning={"138786",},
}
end

function SpellInfo.GetUnknown_v()
 return {
valor={"40724",},
valor_medal_of_the_first_war={"60054",},
vapor_lock={"136085",},
varothens_brooch={"102664",},
veil_of_lies={"102667",},
velocity={"107982","109709","109711","109787","109789","109802","109804","126599",},
vengeance={"39445",},
vengeful_wisp={"91075",},
venomhide_poison={"14795",},
venomous_totem={"23726",},
venture_company_beatdown={"51348","51351","51353","51360",},
very_happy={"43776",},
vessel_of_the_naaru={"45064",},
vial_of_ichorous_blood={"126270",},
vicious={"148903",},
victorious={"96907","97120",},
victory={"96908","97121",},
vigilance_of_the_colossus={"33089","33090",},
virmens_bite={"105697",},
viseclaw_carapace={"118874",},
visionary={"90854",},
visions_of_insanity={"127230",},
visions_of_the_past={"17623",},
volatile_power={"67736","67744",},
volatility={"67735","67743",},
volcanic_destruction={"89091",},
volcanic_power={"79476",},
}
end

function SpellInfo.GetUnknown_1()
 return { }
end

function SpellInfo.GetUnknown_0()
 return { }
end

function SpellInfo.GetUnknown_3()
 return { }
end

function SpellInfo.GetUnknown_2()
 return { }
end

function SpellInfo.GetUnknown_5()
 return {
["53_heroic_scenario_loot"]={"142116",},
}
end

function SpellInfo.GetUnknown_4()
 return { }
end

